diff --git a/pgadmin/dd/dditems/figures/ddColumnFigure.cpp b/pgadmin/dd/dditems/figures/ddColumnFigure.cpp
index 6f89f99..d17cbe6 100644
--- a/pgadmin/dd/dditems/figures/ddColumnFigure.cpp
+++ b/pgadmin/dd/dditems/figures/ddColumnFigure.cpp
@@ -26,26 +26,27 @@
 #include "dd/dditems/figures/ddTableFigure.h"
 #include "dd/dditems/figures/ddRelationshipItem.h"
 
-ddColumnFigure::ddColumnFigure(wxString &columnName, ddTableFigure *owner, ddRelationshipItem *sourceFk)
+void ddColumnFigure::Init(wxString &columnName, ddTableFigure *owner, ddRelationshipItem *sourceFk)
 {
+	setKindId(DDCOLUMNFIGURE);
 	fkSource = sourceFk;
 	usedAsFkDestFor = NULL;
 	setOwnerTable(owner);
 	deactivateGenFkName(); //initializae by default at not generate auto fk name
 	columnText = new ddTextTableItemFigure(columnName, dt_null, this);
-	leftImage = new ddColumnKindIcon(this);
-	centerImage = new ddColumnOptionIcon(this);
+	kindImage = new ddColumnKindIcon(this);
+	optionImage = new ddColumnOptionIcon(this);
 
 
 	//Initialize displaybox and image coords
 	basicDisplayBox.SetPosition(wxPoint(0, 0));
 	basicDisplayBox.SetSize( columnText->displayBox().GetSize());
-	if(leftImage && centerImage)
+	if(kindImage && optionImage)
 	{
-		basicDisplayBox.width += leftImage->displayBox().GetSize().GetWidth() + 3;
-		basicDisplayBox.width += centerImage->displayBox().GetSize().GetWidth() + 3;
-		columnText->displayBox().x += leftImage->displayBox().GetSize().GetWidth() + 2;
-		columnText->displayBox().x += centerImage->displayBox().GetSize().GetWidth() + 3;
+		basicDisplayBox.width += kindImage->displayBox().GetSize().GetWidth() + 3;
+		basicDisplayBox.width += optionImage->displayBox().GetSize().GetWidth() + 3;
+		columnText->displayBox().x += kindImage->displayBox().GetSize().GetWidth() + 2;
+		columnText->displayBox().x += optionImage->displayBox().GetSize().GetWidth() + 3;
 	}
 	else
 	{
@@ -55,25 +56,59 @@ ddColumnFigure::ddColumnFigure(wxString &columnName, ddTableFigure *owner, ddRel
 
 	//Set Value default Attributes
 	fontAttribute->font().SetPointSize(owner->fontAttribute->font().GetPointSize());
+
+}
+
+ddColumnFigure::ddColumnFigure(wxString &columnName, ddTableFigure *owner, ddRelationshipItem *sourceFk)
+{
+	Init(columnName, owner, sourceFk);
+}
+
+//Constructor for creation of existing column from storage
+ddColumnFigure::ddColumnFigure(wxString &columnName, ddTableFigure *owner, ddColumnOptionType option, bool isGenFk, bool isPkColumn, wxString colDataType, int p, int s, int ukIdx, ddRelationshipItem *srcFk, ddRelationshipItem *usedAsFk )
+{
+	Init(columnName, owner, srcFk);
+	setColumnOption(option);
+	generateFkName = isGenFk;
+	kindImage->setPrimaryKey(isPkColumn);
+
+	int dtype = columnText->dataTypes().Index(colDataType,false);
+	if(dtype != wxNOT_FOUND)
+	{
+		setDataType((ddDataType)dtype);
+	}
+	else
+	{
+		wxMessageBox(_("Not compatible data type for column found"));
+	}
+
+	columnText->setPrecision(p);
+	columnText->setScale(s);
+	kindImage->setUkIndex(ukIdx);
+
+	usedAsFkDestFor = usedAsFk;
+	if(ownerTable)
+		ownerTable->updateTableSize();
+
 }
 
 ddColumnFigure::~ddColumnFigure()
 {
 	if(columnText)
 		delete columnText;
-	if(leftImage)
-		delete leftImage;
-	if(centerImage)
-		delete centerImage;
+	if(kindImage)
+		delete kindImage;
+	if(optionImage)
+		delete optionImage;
 }
 
 void ddColumnFigure::basicMoveBy(int x, int y)
 {
 	wxhdAbstractFigure::basicMoveBy(x, y);
-	if(leftImage)
-		leftImage->moveBy(x, y);
-	if(centerImage)
-		centerImage->moveBy(x, y);
+	if(kindImage)
+		kindImage->moveBy(x, y);
+	if(optionImage)
+		optionImage->moveBy(x, y);
 	columnText->moveBy(x, y);
 }
 
@@ -82,20 +117,20 @@ void ddColumnFigure::moveTo(int x, int y)
 	wxhdAbstractFigure::moveTo(x, y);
 	int distance = 0;
 
-	if(leftImage)
+	if(kindImage)
 	{
-		leftImage->moveTo(x, y);
-		distance += leftImage->displayBox().GetSize().GetWidth() + 3;
+		kindImage->moveTo(x, y);
+		distance += kindImage->displayBox().GetSize().GetWidth() + 3;
 	}
 	else
 	{
 		distance += 11; //value = 8 + 3
 	}
 
-	if(centerImage)
+	if(optionImage)
 	{
-		centerImage->moveTo(x + distance, y);
-		distance += centerImage->displayBox().GetSize().GetWidth() + 3;
+		optionImage->moveTo(x + distance, y);
+		distance += optionImage->displayBox().GetSize().GetWidth() + 3;
 	}
 	else
 	{
@@ -112,9 +147,9 @@ bool ddColumnFigure::containsPoint(int x, int y)
 
 	if(columnText->containsPoint(x, y))
 		out = true;
-	else if(leftImage->containsPoint(x, y))
+	else if(kindImage->containsPoint(x, y))
 		out = true;
-	else if(centerImage->containsPoint(x, y))
+	else if(optionImage->containsPoint(x, y))
 		out = true;
 
 	return out;
@@ -128,19 +163,19 @@ wxhdRect &ddColumnFigure::getBasicDisplayBox()
 void ddColumnFigure::basicDraw(wxBufferedDC &context, wxhdDrawingView *view)
 {
 	columnText->draw(context, view);
-	if(leftImage)
-		leftImage->draw(context, view);
-	if(centerImage)
-		centerImage->draw(context, view);
+	if(kindImage)
+		kindImage->draw(context, view);
+	if(optionImage)
+		optionImage->draw(context, view);
 }
 
 void ddColumnFigure::basicDrawSelected(wxBufferedDC &context, wxhdDrawingView *view)
 {
 	columnText->drawSelected(context, view);
-	if(leftImage)
-		leftImage->drawSelected(context, view);
-	if(centerImage)
-		centerImage->drawSelected(context, view);
+	if(kindImage)
+		kindImage->drawSelected(context, view);
+	if(optionImage)
+		optionImage->drawSelected(context, view);
 }
 
 wxhdIFigure *ddColumnFigure::findFigure(int x, int y)
@@ -150,11 +185,11 @@ wxhdIFigure *ddColumnFigure::findFigure(int x, int y)
 	if(columnText->containsPoint(x, y))
 		out = columnText;
 
-	if(leftImage && leftImage->containsPoint(x, y))
-		out = leftImage;
+	if(kindImage && kindImage->containsPoint(x, y))
+		out = kindImage;
 
-	if(centerImage && centerImage->containsPoint(x, y))
-		out = centerImage;
+	if(optionImage && optionImage->containsPoint(x, y))
+		out = optionImage;
 
 	return out;
 }
@@ -167,10 +202,10 @@ wxhdITool *ddColumnFigure::CreateFigureTool(wxhdDrawingEditor *editor, wxhdITool
 wxhdIFigure *ddColumnFigure::getFigureAt(int pos)
 {
 	if(pos == 0)
-		return (wxhdIFigure *) leftImage;
+		return (wxhdIFigure *) kindImage;
 
 	if(pos == 1)
-		return (wxhdIFigure *) centerImage;
+		return (wxhdIFigure *) optionImage;
 
 	if(pos == 2)
 		return (wxhdIFigure *) columnText;
@@ -190,12 +225,12 @@ void ddColumnFigure::setOwnerTable(ddTableFigure *table)
 
 void ddColumnFigure::displayBoxUpdate()
 {
-	if(leftImage && centerImage)
+	if(kindImage && optionImage)
 	{
 		columnText->displayBoxUpdate();
 		basicDisplayBox.width = columnText->displayBox().GetSize().GetWidth();
-		basicDisplayBox.width += leftImage->displayBox().GetSize().GetWidth() + 3;
-		basicDisplayBox.width += centerImage->displayBox().GetSize().GetWidth() + 3;
+		basicDisplayBox.width += kindImage->displayBox().GetSize().GetWidth() + 3;
+		basicDisplayBox.width += optionImage->displayBox().GetSize().GetWidth() + 3;
 	}
 	else
 	{
@@ -206,69 +241,69 @@ void ddColumnFigure::displayBoxUpdate()
 
 bool ddColumnFigure::isNull()
 {
-	return centerImage->getOption() == null;
+	return optionImage->getOption() == null;
 }
 
 bool ddColumnFigure::isNotNull()
 {
-	return centerImage->getOption() == notnull;
+	return optionImage->getOption() == notnull;
 }
 
 bool ddColumnFigure::isNone()
 {
-	return leftImage->isNone();
+	return kindImage->isNone();
 }
 
 bool ddColumnFigure::isPrimaryKey()
 {
-	return leftImage->isPrimaryKey();
+	return kindImage->isPrimaryKey();
 }
 
 void ddColumnFigure::disablePrimaryKey()
 {
-	leftImage->disablePrimaryKey();
+	kindImage->disablePrimaryKey();
 }
 
 void ddColumnFigure::enablePrimaryKey()
 {
-	leftImage->enablePrimaryKey();
+	kindImage->enablePrimaryKey();
 }
 
 bool ddColumnFigure::isUniqueKey()
 {
-	return leftImage->isUniqueKey();
+	return kindImage->isUniqueKey();
 }
 
 bool ddColumnFigure::isUniqueKey(int uniqueIndex)
 {
-	return leftImage->isUniqueKey(uniqueIndex);
+	return kindImage->isUniqueKey(uniqueIndex);
 }
 
 bool ddColumnFigure::isPlain()
 {
-	return leftImage->isNone();
+	return kindImage->isNone();
 }
 
 void ddColumnFigure::setColumnKindToNone()
 {
-	leftImage->disablePrimaryKey();
-	leftImage->disableUniqueKey();
+	kindImage->disablePrimaryKey();
+	kindImage->disableUniqueKey();
 	//Foreign key cannot be changed by set / toggle functions
 }
 
 void ddColumnFigure::setRightIconForColumn()
 {
-	leftImage->setRightIconForColumn();
+	kindImage->setRightIconForColumn();
 }
 
 void ddColumnFigure::toggleColumnKind(ddColumnType type, wxhdDrawingView *view)
 {
-	leftImage->toggleColumnKind(type, view);
+	kindImage->toggleColumnKind(type, view);
 }
 
 void ddColumnFigure::setColumnOption(ddColumnOptionType type)
 {
-	centerImage->changeIcon(type);
+	optionImage->changeIcon(type);
 }
 
 wxString &ddColumnFigure::getColumnName(bool datatype)
@@ -288,24 +323,17 @@ int ddColumnFigure::getScale()
 
 int ddColumnFigure::getUniqueConstraintIndex()
 {
-	return leftImage->getUniqueConstraintIndex();
+	return kindImage->getUniqueConstraintIndex();
 }
 
 void ddColumnFigure::setUniqueConstraintIndex(int i)
 {
-	leftImage->setUniqueConstraintIndex(i);
+	kindImage->setUniqueConstraintIndex(i);
 }
 
-/*
-ddColumnType ddColumnFigure::getColumnKind()
-{
-	return leftImage->getKind();
-}
-*/
-
 ddColumnOptionType ddColumnFigure::getColumnOption()
 {
-	return centerImage->getOption();
+	return optionImage->getOption();
 }
 
 ddDataType ddColumnFigure::getDataType()
@@ -364,7 +392,7 @@ void ddColumnFigure::setAsUserCreatedFk(ddRelationshipItem *relatedFkItem)
 	//Now fix icons of kind of column
 	if(relatedFkItem == NULL)
 	{
-		leftImage->setRightIconForColumn();
+		kindImage->setRightIconForColumn();
 	}
 }
 
@@ -456,3 +484,9 @@ bool ddColumnFigure::validateColumn(wxString &errors)
 
 	return out;
 }
+
+//Used by persistence classes
+void ddColumnFigure::setFkSource(ddRelationshipItem *newColumn)
+{
+	fkSource = newColumn;
+}
\ No newline at end of file
diff --git a/pgadmin/dd/dditems/figures/ddColumnKindIcon.cpp b/pgadmin/dd/dditems/figures/ddColumnKindIcon.cpp
index 144cf6f..fd3934d 100644
--- a/pgadmin/dd/dditems/figures/ddColumnKindIcon.cpp
+++ b/pgadmin/dd/dditems/figures/ddColumnKindIcon.cpp
@@ -35,6 +35,7 @@
 
 ddColumnKindIcon::ddColumnKindIcon(ddColumnFigure *owner)
 {
+	setKindId(DDCOLUMNKINDICON);
 	ownerColumn = owner;
 	// Initialize with an image to allow initial size calculations
 	icon = wxBitmap(*ddprimarykey_png_img);
@@ -417,3 +418,16 @@ void ddColumnKindIcon::enablePrimaryKey()
 	getOwnerColumn()->getOwnerTable()->updateFkObservers();
 	setRightIconForColumn();
 }
+
+//Only used for figures created from storage
+void ddColumnKindIcon::setPrimaryKey(bool value)
+{
+	isPk=value;
+	setRightIconForColumn();
+}
+
+//Only used for figures created from storage
+void ddColumnKindIcon::setUkIndex(int ukIdx)
+{
+	ukIndex=ukIdx;
+}
\ No newline at end of file
diff --git a/pgadmin/dd/dditems/figures/ddColumnOptionIcon.cpp b/pgadmin/dd/dditems/figures/ddColumnOptionIcon.cpp
index 0f1a06a..685f744 100644
--- a/pgadmin/dd/dditems/figures/ddColumnOptionIcon.cpp
+++ b/pgadmin/dd/dditems/figures/ddColumnOptionIcon.cpp
@@ -27,6 +27,7 @@
 
 ddColumnOptionIcon::ddColumnOptionIcon(ddColumnFigure *owner)
 {
+	setKindId(DDCOLUMNOPTIONICON);
 	ownerColumn = owner;
 	colOption = null;
 	icon = wxBitmap(*ddnull_png_img);
diff --git a/pgadmin/dd/dditems/figures/ddRelationshipFigure.cpp b/pgadmin/dd/dditems/figures/ddRelationshipFigure.cpp
index 3c47978..d355202 100644
--- a/pgadmin/dd/dditems/figures/ddRelationshipFigure.cpp
+++ b/pgadmin/dd/dditems/figures/ddRelationshipFigure.cpp
@@ -31,6 +31,7 @@ ddRelationshipFigure::ddRelationshipFigure():
 	fkMandatory = true;
 	fkOneToMany = true;
 	fkIdentifying = false;
+	matchSimple = false;
 	ukIndex = -1;
 	disconnectedEndTable = NULL;
 	paintingFkColumns = false;
@@ -163,7 +164,7 @@ void ddRelationshipFigure::updateForeignKey()
 						{
 							if(fkColumnRelItem->isAutomaticallyGenerated()) //don't remove from fk_dest table fk column created from existing column, just mark now as not foreign key
 							{
-								fkColumnRelItem->destinationTable->removeColumn(fkColumnRelItem->fkColumn);
+								fkColumnRelItem->getDestinationTable()->removeColumn(fkColumnRelItem->fkColumn);
 							}
 							else
 							{
@@ -208,7 +209,7 @@ void ddRelationshipFigure::updateForeignKey()
 						{
 							if(fkColumnRelItem->isAutomaticallyGenerated()) //don't remove from fk_dest table fk column created from existing column, just mark now as not foreign key
 							{
-								fkColumnRelItem->destinationTable->removeColumn(fkColumnRelItem->fkColumn);
+								fkColumnRelItem->getDestinationTable()->removeColumn(fkColumnRelItem->fkColumn);
 							}
 							else
 							{
@@ -606,12 +607,12 @@ void ddRelationshipFigure::removeForeignKeys()
 			{
 				wxString key = it->first;
 				fkColumnRelItem = it->second;
-				if(fkColumnRelItem->destinationTable->includes(fkColumnRelItem->fkColumn))
+				if(fkColumnRelItem->getDestinationTable()->includes(fkColumnRelItem->fkColumn))
 				{
 					//Remove fk column only if that column is automatically generated
 					if(fkColumnRelItem->isAutomaticallyGenerated())
 					{
-						fkColumnRelItem->destinationTable->removeColumn(fkColumnRelItem->fkColumn);
+						fkColumnRelItem->getDestinationTable()->removeColumn(fkColumnRelItem->fkColumn);
 					} //is an existing column use as fk
 					else
 					{
@@ -826,4 +827,45 @@ void ddRelationshipFigure::changeFkOSTextColor(wxColour originalColour, wxColour
 		}
 	}
 
+}
+
+//Used by persistence classes
+int ddRelationshipFigure::getUkIndex()
+{
+	return ukIndex;
+}
+
+//Used by persistence classes
+actionKind ddRelationshipFigure::getOnUpdateAction()
+{
+	return onUpdateAction;
+}
+
+//Used by persistence classes
+actionKind ddRelationshipFigure::getOnDeleteAction()
+{
+	return onDeleteAction;
+}
+
+//Used by persistence classes
+bool ddRelationshipFigure::getMatchSimple()
+{
+	return matchSimple;
+}
+
+//Used by persistence classes
+void ddRelationshipFigure::initRelationValues( ddTableFigure *source, ddTableFigure *destination, int ukIdx, wxString constraint, actionKind onUpdate, actionKind onDelete, bool simpleMatch, bool identifying, bool oneToMany, bool mandatory, bool fromPk )
+{
+	ukIndex = ukIdx;
+	fkFromPk=fromPk;
+	fkMandatory=mandatory;
+	fkOneToMany=oneToMany;
+	fkIdentifying=identifying;
+	matchSimple=simpleMatch;
+	onUpdateAction=onUpdate;
+	onDeleteAction=onDelete;
+	constraintName = constraint;
+
+	wxhdLineConnection::connectStart(source->connectorAt(getStartPoint().x,getStartPoint().y));
+	wxhdLineConnection::connectEnd(destination->connectorAt(getEndPoint().x,getEndPoint().y));
 }
\ No newline at end of file
diff --git a/pgadmin/dd/dditems/figures/ddRelationshipItem.cpp b/pgadmin/dd/dditems/figures/ddRelationshipItem.cpp
index 8e7aea0..20fcb8a 100644
--- a/pgadmin/dd/dditems/figures/ddRelationshipItem.cpp
+++ b/pgadmin/dd/dditems/figures/ddRelationshipItem.cpp
@@ -21,6 +21,13 @@
 #include "dd/wxhotdraw/main/wxhdDrawingView.h"
 #include "dd/dditems/utilities/ddDataType.h"
 
+ddRelationshipItem::ddRelationshipItem()
+{
+	ownerRel = NULL;
+	original = NULL;
+	destinationTable = NULL;
+}
+
 ddRelationshipItem::ddRelationshipItem(ddRelationshipFigure *owner, ddColumnFigure *originalColumn, ddTableFigure *destination, ddColumnOptionType type, ddColumnType colType, ddColumnFigure *existingColumn)
 {
 	ownerRel = owner;
@@ -54,6 +61,24 @@ ddRelationshipItem::~ddRelationshipItem()
 {
 }
 
+void ddRelationshipItem::initRelationshipItemValues(ddRelationshipFigure *owner, ddTableFigure *destination, bool fromExistingColumn, ddColumnFigure *fkCol, ddColumnFigure *sourceCol, wxString initialColName, wxString initialAliasName )
+{
+	ownerRel = owner;
+	destinationTable = destination;
+	fkIsAutoGenerated = fromExistingColumn;
+	original = sourceCol;
+	fkColumn = fkCol;
+	originalStartColName = initialColName;
+	originalShortName = initialAliasName;
+	if(fkIsAutoGenerated)
+		fkColumn->setFkSource(this);
+	else
+		fkColumn->setAsUserCreatedFk(this);		
+	
+	fkColumn->setRightIconForColumn();
+}
+
+
 bool ddRelationshipItem::isAutomaticallyGenerated()
 {
 	return fkIsAutoGenerated;
diff --git a/pgadmin/dd/dditems/figures/ddTableFigure.cpp b/pgadmin/dd/dditems/figures/ddTableFigure.cpp
index 075efd2..df6bce3 100644
--- a/pgadmin/dd/dditems/figures/ddTableFigure.cpp
+++ b/pgadmin/dd/dditems/figures/ddTableFigure.cpp
@@ -57,8 +57,7 @@ All figures title, colums, indexes are store at same array to improve performanc
 	[maxIdxIndex] = last index index
 */
 
-ddTableFigure::ddTableFigure(wxString tableName, int x, int y, wxString shortName):
-	wxhdCompositeFigure()
+void ddTableFigure::Init(wxString tableName, int x, int y, wxString shortName)
 {
 	setKindId(DDTABLEFIGURE);
 	internalPadding = 2;
@@ -130,6 +129,29 @@ ddTableFigure::ddTableFigure(wxString tableName, int x, int y, wxString shortNam
 	updateTableSize();
 }
 
+ddTableFigure::ddTableFigure(wxString tableName, int x, int y, wxString shortName):
+	wxhdCompositeFigure()
+{
+	Init(tableName,x,y,shortName);
+}
+
+//Used by persistence classes
+void ddTableFigure::InitTableValues(wxArrayString UniqueKeysName, wxString primaryKeyName, int bdc, int bdi, int maxcolsi, int minidxsi, int maxidxsi, int colsrs, int colsw, int idxsrs, int idxsw)
+{
+	ukNames = UniqueKeysName;
+	pkName = primaryKeyName;
+	beginDrawCols = bdc;
+	beginDrawIdxs = bdi;
+	maxColIndex = maxcolsi;
+	minIdxIndex = minidxsi;
+	maxIdxIndex = maxidxsi;
+	colsRowsSize = colsrs;
+	colsWindow = colsw;
+	idxsRowsSize = idxsrs;
+	idxsWindow = idxsw;
+	updateTableSize();
+}
+
 ddTableFigure::~ddTableFigure()
 {
 	if(scrollbar)
@@ -140,9 +162,9 @@ ddTableFigure::~ddTableFigure()
 	}
 }
 
-bool ddTableFigure::colNameAvailable(wxString name)
+ddColumnFigure* ddTableFigure::getColByName(wxString name)
 {
-	bool out = true;
+	ddColumnFigure *out = NULL;
 	ddColumnFigure *f;
 
 	wxhdIteratorBase *iterator = figuresEnumerator();
@@ -154,7 +176,7 @@ bool ddTableFigure::colNameAvailable(wxString name)
 		f = (ddColumnFigure *) iterator->Next();
 		if(f->getColumnName(false).IsSameAs(name))
 		{
-			out = false;
+			out = f;
 			break;
 		}
 	}
@@ -179,6 +201,20 @@ void ddTableFigure::addColumn(ddColumnFigure *column)
 	colsRowsSize++;
 	calcRectsAreas();
 	recalculateColsPos();
+	updateTableSize();
+}
+
+//WARNING: Function should be called on a table generated from a storage
+void ddTableFigure::syncPositionsAfterLoad()
+{
+	calcRectsAreas();
+	recalculateColsPos();
+}
+
+//WARNING: Columns SHOULD BE ADDED only using this columns if was created as an image from storage one
+void ddTableFigure::addColumnFromStorage(ddColumnFigure *column)
+{
+	add(column);
 }
 
 void ddTableFigure::removeColumn(ddColumnFigure *column)
diff --git a/pgadmin/dd/dditems/figures/ddTextTableItemFigure.cpp b/pgadmin/dd/dditems/figures/ddTextTableItemFigure.cpp
index 4b15510..e902499 100644
--- a/pgadmin/dd/dditems/figures/ddTextTableItemFigure.cpp
+++ b/pgadmin/dd/dditems/figures/ddTextTableItemFigure.cpp
@@ -29,6 +29,7 @@
 ddTextTableItemFigure::ddTextTableItemFigure(wxString &columnName, ddDataType dataType, ddColumnFigure *owner):
 	wxhdSimpleTextFigure(columnName)
 {
+	setKindId(DDTEXTTABLEITEMFIGURE);
 	ownerTable = NULL; //table name item is the only one case of use of this variable
 	oneTimeNoAlias = false;
 	columnType = dataType;
diff --git a/pgadmin/dd/dditems/figures/module.mk b/pgadmin/dd/dditems/figures/module.mk
index e7130ff..53a0817 100644
--- a/pgadmin/dd/dditems/figures/module.mk
+++ b/pgadmin/dd/dditems/figures/module.mk
@@ -21,3 +21,5 @@ pgadmin3_SOURCES += \
 
 EXTRA_DIST += \
 	$(srcdir)/dd/dditems/figures/module.mk
+
+include $(srcdir)/dd/dditems/figures/xml/module.mk
diff --git a/pgadmin/dd/dditems/figures/xml/ddXmlStorage.cpp b/pgadmin/dd/dditems/figures/xml/ddXmlStorage.cpp
new file mode 100644
index 0000000..f3683d9
--- /dev/null
+++ b/pgadmin/dd/dditems/figures/xml/ddXmlStorage.cpp
@@ -0,0 +1,1522 @@
+//////////////////////////////////////////////////////////////////////////
+//
+// pgAdmin III - PostgreSQL Tools
+//
+// Copyright (C) 2002 - 2011, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+// ddXmlStorage.cpp - Database designer class for storing / loading objects
+//
+//////////////////////////////////////////////////////////////////////////
+
+#include "pgAdmin3.h"
+
+
+#include <libxml/xmlwriter.h>
+#include <libxml/xmlreader.h>
+
+// App headers
+#include "dd/dditems/figures/xml/ddXmlStorage.h"
+#include "dd/dditems/utilities/ddDataType.h"
+#include "dd/wxhotdraw/figures/wxhdIFigure.h"
+#include "dd/dditems/figures/ddTableFigure.h"
+#include "dd/dditems/figures/ddColumnFigure.h"
+#include "dd/dditems/figures/ddRelationshipFigure.h"
+#include "dd/dditems/figures/ddRelationshipItem.h"
+#include "dd/dditems/figures/ddRelationshipTerminal.h"
+#include "dd/dditems/figures/ddColumnKindIcon.h"
+#include "dd/dditems/figures/ddColumnOptionIcon.h"
+#include "dd/ddmodel/ddDatabaseDesign.h"
+
+ddDatabaseDesign* ddXmlStorage::design = NULL;
+
+ddXmlStorage::ddXmlStorage():
+	wxhdStorage()
+{
+	design = NULL;
+}
+
+#define XML_FROM_WXSTRING(s) ((xmlChar *)(const char *)s.mb_str(wxConvUTF8))
+#define WXSTRING_FROM_XML(s) wxString((char *)s, wxConvUTF8)
+
+void ddXmlStorage::setModel(ddDatabaseDesign *sourceDesign)
+{
+	design = sourceDesign;
+}
+
+void ddXmlStorage::StartModel(xmlTextWriterPtr writer, ddDatabaseDesign *sourceDesign)
+{
+	int tmp;
+	design = sourceDesign;
+	//<!DOCTYPE MODEL ...pgRelationModel.dtd>
+	// but add all dtd to doc instead of url
+	wxString name = _("MODEL");
+	wxString pubid = _("pgrm");
+	wxString uri = _("pgrm");
+
+	xmlTextWriterWriteDTD(writer,XML_FROM_WXSTRING(name),NULL,NULL,XML_FROM_WXSTRING(getModelDTD()));
+	//<!ELEMENT MODEL (TABLE+)>
+	tmp = xmlTextWriterStartElement(writer, BAD_CAST "MODEL");
+	processResult(tmp);
+
+}
+
+void ddXmlStorage::EndModel( xmlTextWriterPtr writer)
+{
+	//Close MODEL Element
+	xmlTextWriterEndElement(writer);
+	design = NULL;
+}
+
+bool ddXmlStorage::Write(xmlTextWriterPtr writer, wxhdIFigure *figure)
+{
+	if(design==NULL)
+	{
+		wxMessageBox(_("Error saving model: no source model"), _("Error"), wxICON_ERROR);
+		return false;
+	}
+
+	switch(figure->getKindId())
+	{
+		case DDCOLUMNOPTIONICON:
+			WriteLocal(writer,(ddColumnOptionIcon *)figure);
+			break;
+		case DDCOLUMNKINDICON:
+			WriteLocal(writer,(ddColumnKindIcon *)figure);
+			break;
+		case DDTEXTTABLEITEMFIGURE:
+			WriteLocal(writer,(ddTextTableItemFigure *)figure);
+			break;
+		case DDCOLUMNFIGURE:
+			WriteLocal(writer,(ddColumnFigure *)figure);
+			break;
+		case DDTABLEFIGURE:
+			WriteLocal(writer,(ddTableFigure *)figure);
+			break;
+		case DDRELATIONSHIPFIGURE:
+			WriteLocal(writer,(ddRelationshipFigure *)figure);
+			break;
+	}
+	return true;
+}
+
+bool ddXmlStorage::processResult(int value)
+{
+    if (value < 0) {
+        wxMessageBox(_("Failed to write the model file"), _("Error"), wxICON_ERROR);
+		return false;
+    }
+	return true;
+}
+
+void ddXmlStorage::WriteLocal(xmlTextWriterPtr writer, ddColumnOptionIcon *figure)
+{
+    int tmp;
+
+	//At COLUMN Element
+	//<!ELEMENT OPTION (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "OPTION","%d", figure->getOption() );
+	processResult(tmp);
+}
+
+void ddXmlStorage::WriteLocal(xmlTextWriterPtr writer, ddColumnKindIcon *figure)
+{
+    int tmp;
+
+	//At COLUMN Element
+	//<!ELEMENT UKINDEX (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "UKINDEX","%d", figure->getUniqueConstraintIndex());
+	processResult(tmp);
+
+	//<!ELEMENT ISPK (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "ISPK","%c", figure->isPrimaryKey() ? 'T' : 'F' );
+	processResult(tmp);
+}
+
+void ddXmlStorage::WriteLocal( xmlTextWriterPtr writer, ddTextTableItemFigure *figure)
+{
+	int tmp;
+
+	//<!ELEMENT COLUMNTYPE (#PCDATA)> 
+	wxString ddType = figure->dataTypes()[figure->getDataType()];
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "COLUMNTYPE","%s",XML_FROM_WXSTRING(ddType));
+	processResult(tmp);	
+
+	if(figure->getPrecision() != -1)
+	{
+		//<!ELEMENT PRECISION (#PCDATA)>
+		tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "PRECISION","%d", figure->getPrecision());
+		processResult(tmp);	
+	}
+
+	if(figure->getScale() != -1)
+	{
+		//<!ELEMENT SCALE (#PCDATA)>
+		tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "SCALE","%d", figure->getScale());
+		processResult(tmp);	
+	}
+
+}
+
+void ddXmlStorage::WriteLocal( xmlTextWriterPtr writer, ddColumnFigure *figure)
+{
+	int tmp;
+	
+	//At TABLE Element
+	//<!ELEMENT COLUMN (Attribute+,OPTION,UKINDEX,ISPK,SHOWDATATYPE,SHOWALIAS,COLUMNTYPE,PRECISION?,SCALE?,GENERATEFKNAME)>	
+	tmp = xmlTextWriterStartElement(writer, BAD_CAST "COLUMN");
+	
+	//<!ELEMENT NAME (#PCDATA)>
+	wxString columnName = figure->getColumnName(false);
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "NAME","%s", XML_FROM_WXSTRING(columnName));
+	processResult(tmp);	
+
+	//At Column Element
+	Write(writer, figure->getOptionImage());
+	Write(writer, figure->getKindImage());
+	Write(writer, figure->getColumnText());
+	
+	//At Column Element
+	//<!ELEMENT GENERATEFKNAME (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "GENERATEFKNAME","%c",figure->isFkNameGenerated() ?  'T' : 'F' );
+	processResult(tmp);	
+
+	//Close COLUMN Element
+	xmlTextWriterEndElement(writer);
+	processResult(tmp);	
+}
+
+void ddXmlStorage::WriteLocal( xmlTextWriterPtr writer, ddTableFigure *figure)
+{
+	int tmp;
+	
+	//At TABLES ELEMENT
+	//<!ELEMENT TABLE (Attribute+,COLUMNS,UKNAMES?, PKNAME, BEGINDRAWCOLS, BEGINDRAWIDXS, MAXCOLINDEX, MINIDXINDEX, MAXIDXINDEX, COLSROWSSIZE, COLSWINDOW,  IDXSROWSSIZE, IDXSWINDOW )>
+	tmp = xmlTextWriterStartElement(writer, BAD_CAST "TABLE");
+	processResult(tmp);	
+	
+	//<!ATTLIST TABLE	TableID ID #REQUIRED > --> ddTableFigure ID
+	wxString TableId = design->getTableId(figure->getTableName());
+	tmp = xmlTextWriterWriteAttribute(writer, BAD_CAST "TableID", XML_FROM_WXSTRING(TableId) );
+	processResult(tmp);
+
+		//Start POINT <!ELEMENT POINT (X,Y)>
+		tmp = xmlTextWriterStartElement(writer, BAD_CAST "POINT");
+		//<!ELEMENT X (#PCDATA)>
+		tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "X","%d", figure->getBasicDisplayBox().x);
+		processResult(tmp);
+		//<!ELEMENT Y (#PCDATA)>
+		tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "Y","%d", figure->getBasicDisplayBox().y);
+		processResult(tmp);
+		//Close POINT Element
+		xmlTextWriterEndElement(writer);
+
+		//Add Table Title
+		ddColumnFigure *f;
+		wxhdIteratorBase *iterator = figure->figuresEnumerator();
+		iterator->Next(); //First figure is main rect
+
+		//<!ELEMENT TITLE (NAME,ALIAS?)>
+		tmp = xmlTextWriterStartElement(writer, BAD_CAST "TITLE");
+		processResult(tmp);	
+		f = (ddColumnFigure *) iterator->Next();
+
+		//<!ELEMENT NAME (#PCDATA)>
+		tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "NAME","%s",XML_FROM_WXSTRING(figure->getTableName()));
+		processResult(tmp);	
+
+		//<!ELEMENT ALIAS (#PCDATA)>
+		if(figure->getShortTableName().Length()>0)
+		{
+			//<!ELEMENT ALIAS (#PCDATA)>
+			tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "ALIAS","%s", XML_FROM_WXSTRING(figure->getShortTableName()));
+			processResult(tmp);	
+		}
+
+		//Close TITLE Element
+		xmlTextWriterEndElement(writer);
+
+		if(figure->getUkConstraintsNames().Count()>0)
+		{
+			//<!ELEMENT UKNAMES (UKNAME+)>
+			tmp = xmlTextWriterStartElement(writer, BAD_CAST "UKNAMES");
+			processResult(tmp);	
+
+			//<!ELEMENT UKNAME (#PCDATA)>  one for each name
+			int i, last = figure->getUkConstraintsNames().Count();
+			wxString ukName;
+			for(i = 0; i < last; i++)
+			{
+				ukName = figure->getUkConstraintsNames()[i];
+				tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "UKNAME","%s", XML_FROM_WXSTRING(ukName));
+				processResult(tmp);	
+			}
+			
+			//Close UKNAMES Element
+			xmlTextWriterEndElement(writer);
+		}
+
+	//<!ELEMENT PKNAME (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "PKNAME","%s",XML_FROM_WXSTRING(figure->getPkConstraintName()));
+	processResult(tmp);
+			
+	//<!ELEMENT BEGINDRAWCOLS (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "BEGINDRAWCOLS","%d",figure->getBeginDrawCols());
+	processResult(tmp);
+
+	//<!ELEMENT BEGINDRAWIDXS (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "BEGINDRAWIDXS","%d",figure->getBeginDrawIdxs());
+	processResult(tmp);
+
+	//<!ELEMENT MAXCOLINDEX (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "MAXCOLINDEX","%d",figure->getMaxColIndex());
+	processResult(tmp);
+
+	//<!ELEMENT MINIDXINDEX (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "MINIDXINDEX","%d",figure->getMinIdxIndex());
+	processResult(tmp);
+
+	//<!ELEMENT MAXIDXINDEX (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "MAXIDXINDEX","%d",figure->getMaxIdxIndex());
+	processResult(tmp);
+
+	//<!ELEMENT COLSROWSSIZE (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "COLSROWSSIZE","%d",figure->getColsRowsSize());
+	processResult(tmp);
+
+	//<!ELEMENT COLSWINDOW (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "COLSWINDOW","%d",figure->getColsWindow());
+	processResult(tmp);
+
+	//<!ELEMENT IDXSROWSSIZE (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "IDXSROWSSIZE","%d",figure->getIdxsRowsSize());
+	processResult(tmp);
+
+	//<!ELEMENT IDXSWINDOW (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "IDXSWINDOW","%d",figure->getIdxsWindow());
+	processResult(tmp);
+
+	//<!ELEMENT COLUMNS (COLUMN*)>
+	tmp = xmlTextWriterStartElement(writer, BAD_CAST "COLUMNS");
+	processResult(tmp);	
+
+	//ALL COLUMN ELEMENT
+	while(iterator->HasNext())
+	{
+		f = (ddColumnFigure *) iterator->Next();
+		Write(writer,f);
+	}
+	delete iterator;
+	
+	//Close COLUMNS Element
+	xmlTextWriterEndElement(writer);
+
+	//Close TABLE Element
+	xmlTextWriterEndElement(writer);
+
+}
+
+
+void ddXmlStorage::WriteLocal( xmlTextWriterPtr writer, ddRelationshipFigure *figure)
+{
+	int tmp;
+
+	//At RELATIONSHIPS Element
+	//<!ELEMENT RELATIONSHIP (Attributes*, POINTS, RELATIONITEMS, UKINDEX, NAME, ONUPDATE, ONDELETE, MATCHSIMPLE, IDENTIFYING, ONETOMANY, MANDATORY, FKFROMPK)>
+	tmp = xmlTextWriterStartElement(writer, BAD_CAST "RELATIONSHIP");
+
+	//<!ATTLIST RELATIONITEM SourceTableID IDREF #REQUIRED > --> ddTableFigure ID
+	wxString TableId = design->getTableId(figure->getStartTable()->getTableName());
+	tmp = xmlTextWriterWriteAttribute(writer, BAD_CAST "SourceTableID", XML_FROM_WXSTRING(TableId) );
+	processResult(tmp);
+	
+	//<!ATTLIST RELATIONSHIP DestTableID IDREF #REQUIRED >	
+	TableId = design->getTableId(figure->getEndTable()->getTableName());
+	tmp = xmlTextWriterWriteAttribute(writer, BAD_CAST "DestTableID", XML_FROM_WXSTRING(TableId) );
+	processResult(tmp);
+
+	//At RELATIONSHIP Element
+	//Start POINTS <!ELEMENT POINTS (POINT, POINT,POINT*)>
+	tmp = xmlTextWriterStartElement(writer, BAD_CAST "POINTS");
+	wxhdPoint point;
+	for(int i = 0; i < figure->pointCount(); i++)
+	{
+		//At POINTS Element
+		//Start POINT <!ELEMENT POINT (X,Y)>
+		tmp = xmlTextWriterStartElement(writer, BAD_CAST "POINT");
+			 point = figure->pointAt(i);	
+			//<!ELEMENT X (#PCDATA)>
+			tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "X","%d", point.x);
+			processResult(tmp);
+			//<!ELEMENT Y (#PCDATA)>
+			tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "Y","%d", point.y);
+			processResult(tmp);
+		//Close POINT Element
+		xmlTextWriterEndElement(writer);
+	}
+	
+	//Close POINTS Element
+	xmlTextWriterEndElement(writer);
+
+	//At RELATIONSHIP Element
+	//<!ELEMENT UKINDEX (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "UKINDEX","%d", figure->getUkIndex());
+	processResult(tmp);
+
+	if(figure->getConstraintName().Length() > 0)
+	{
+		//<!ELEMENT NAME (#PCDATA)>
+		wxString name = figure->getConstraintName();
+		tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "NAME","%s", XML_FROM_WXSTRING(name));
+		processResult(tmp);
+	}
+
+	//<!ELEMENT ONUPDATE (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "ONUPDATE","%d", figure->getOnUpdateAction());
+	processResult(tmp);
+
+	//<!ELEMENT ONDELETE (#PCDATA)>
+	tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "ONDELETE","%d", figure->getOnDeleteAction());
+	processResult(tmp);
+
+	//<!ELEMENT MATCHSIMPLE (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "MATCHSIMPLE","%c", figure->getMatchSimple() ? 'T' : 'F' );
+	processResult(tmp);
+
+	//<!ELEMENT IDENTIFYING (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "IDENTIFYING","%c", figure->getIdentifying() ? 'T' : 'F' );
+	processResult(tmp);
+
+	//<!ELEMENT ONETOMANY (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "ONETOMANY","%c", figure->getOneToMany() ? 'T' : 'F' );
+	processResult(tmp);
+
+	//<!ELEMENT MANDATORY (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "MANDATORY","%c", figure->getMandatory() ? 'T' : 'F' );
+	processResult(tmp);
+
+	//<!ELEMENT FKFROMPK (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "FKFROMPK","%c", figure->isForeignKeyFromPk() ? 'T' : 'F' );
+	processResult(tmp);
+
+	//At RELATIONSHIP Element
+	//Start RELATIONITEMS <!ELEMENT RELATIONITEMS (RELATIONITEM*)>
+	tmp = xmlTextWriterStartElement(writer, BAD_CAST "RELATIONITEMS");
+
+	ddRelationshipItem *item;
+	wxString key;
+	columnsHashMap::iterator it;
+	for( it = figure->getItemsHashMap().begin(); it != figure->getItemsHashMap().end(); ++it )
+	{
+			key = it->first;
+			item = it->second;
+			ddXmlStorage::WriteLocal(writer, item);
+	}
+
+	//Close RELATIONITEMS
+	xmlTextWriterEndElement(writer);
+
+	//Close RELATIONSHIP Element
+	xmlTextWriterEndElement(writer);
+}
+
+void ddXmlStorage::WriteLocal( xmlTextWriterPtr writer, ddRelationshipItem *item)
+{
+	int tmp;
+
+	//At RELATIONITEMS Element
+	//<!ELEMENT RELATIONITEM (AUTOGENFK, FKCOLNAME,SOURCECOLNAME, INITIALCOLNAME, INITIALALIASNAME )>
+	tmp = xmlTextWriterStartElement(writer, BAD_CAST "RELATIONITEM");
+	
+	//<!ELEMENT AUTOGENFK (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "AUTOGENFK","%c", item->isAutomaticallyGenerated() ? 'T' : 'F' );
+	processResult(tmp);
+
+	//<!ELEMENT FKCOLNAME (#PCDATA)>
+ 	wxString fkColName = item->fkColumn->getColumnName();
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "FKCOLNAME","%s",XML_FROM_WXSTRING(fkColName));
+	processResult(tmp);	
+
+	//<!ELEMENT SOURCECOLNAME (#PCDATA)>
+ 	wxString sourceColName = item->original->getColumnName();
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "SOURCECOLNAME","%s",XML_FROM_WXSTRING(sourceColName));
+	processResult(tmp);	
+
+	//<!ELEMENT INITIALCOLNAME (#PCDATA)>
+    tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "INITIALCOLNAME","%s",XML_FROM_WXSTRING(item->originalStartColName));
+	processResult(tmp);	
+
+	if(item->originalShortName.Length()>0)
+	{
+		//<!ELEMENT INITIALALIASNAME (#PCDATA)>
+		tmp = xmlTextWriterWriteFormatElement(writer, BAD_CAST "INITIALALIASNAME","%s",XML_FROM_WXSTRING(item->originalShortName));
+		processResult(tmp);	
+	}
+	//Close RELATIONITEM Element
+	xmlTextWriterEndElement(writer);
+}
+
+//This is needed because table doesn't have any explicit order when stored
+void ddXmlStorage::initialModelParse(xmlTextReaderPtr reader)
+{
+	int ret,tmp;
+	xmlChar *value;	
+
+	if(reader!=NULL)
+	{
+		wxString tableName=wxEmptyString, tableAlias=wxEmptyString, TableID;
+		xmlTextReaderSetParserProp( reader, XML_PARSER_VALIDATE, 1 );     
+		ret = xmlTextReaderRead(reader);
+        while (ret == 1)
+		{
+			if(getNodeType(reader)==1 && getNodeName(reader).IsSameAs(_("TABLE"),false)) //libxml 1 for start element
+			{
+				//Look for table ID
+				tmp = xmlTextReaderHasAttributes(reader);
+				TableID=_("TableID");
+				if(tmp)
+				{
+					value = xmlTextReaderGetAttribute(reader,XML_FROM_WXSTRING(TableID));
+				}
+				
+				if(value)
+				{
+						TableID = WXSTRING_FROM_XML(value);
+						xmlFree(value);
+				}
+
+				// --> POINT
+				//<!ELEMENT POINT (X,Y)>
+				int x=0,y=0;
+				wxString tmpInt;
+				tmp = xmlTextReaderRead(reader);	//go POINT
+				tmp = xmlTextReaderRead(reader);	//go X
+				tmp = xmlTextReaderRead(reader);	//go X Value
+				value = xmlTextReaderValue(reader);  //Value of X
+				if(value)
+				{
+					tmpInt = WXSTRING_FROM_XML(value);
+					x = wxAtoi(tmpInt);
+					xmlFree(value);
+				}
+				tmp = xmlTextReaderRead(reader);	//go to /X
+
+				tmp = xmlTextReaderRead(reader);	//go Y
+				tmp = xmlTextReaderRead(reader);	//go Y Value
+				value = xmlTextReaderValue(reader);  //Value of Y
+				if(value)
+				{
+					tmpInt = WXSTRING_FROM_XML(value);
+					y = wxAtoi(tmpInt);
+					xmlFree(value);
+				}
+				tmp = xmlTextReaderRead(reader);	//go to /Y
+				tmp = xmlTextReaderRead(reader);	//go /POINT
+
+				tmp = xmlTextReaderRead(reader);	//go to TITLE
+				tmp = xmlTextReaderRead(reader);	//go to NAME
+				tmp = xmlTextReaderRead(reader);	//go to NAME Value
+				value = xmlTextReaderValue(reader);  //Value of NAME
+				if(value)
+				{
+					tableName = WXSTRING_FROM_XML(value);
+					design->addTableToMapping(TableID,tableName); 
+					xmlFree(value);
+				}
+				tmp = xmlTextReaderRead(reader);	//go to /NAME
+
+				tmp = xmlTextReaderRead(reader);	//go to ALIAS or /TITLE
+				if(getNodeName(reader).IsSameAs(_("ALIAS"),false))
+				{
+					tmp = xmlTextReaderRead(reader);	//go to ALIAS Value	
+					value = xmlTextReaderValue(reader);  //Value of ALIAS
+					if(value)
+					{
+						tableAlias = WXSTRING_FROM_XML(value);
+						xmlFree(value);
+					}
+					tmp = xmlTextReaderRead(reader);	//go to /ALIAS
+					tmp = xmlTextReaderRead(reader);	//go to /TITLE
+				}
+				ddTableFigure *t = new ddTableFigure(tableName, x, y, tableAlias);
+				design->addTable(t);
+			}
+		 ret = xmlTextReaderRead(reader);			
+		}
+
+		// Once the document has been fully parsed, check the validation results
+		if (xmlTextReaderIsValid(reader) != 1)
+		{
+			wxMessageBox(_("Model is not following embedded DTD definition, check it."), _("Error"), wxICON_ERROR);
+			return;
+		}
+	}
+	//reset reader to beginning of stream
+}
+
+bool ddXmlStorage::Read(xmlTextReaderPtr reader)
+{
+	int ret;
+
+	if(reader!=NULL)
+	{
+		xmlTextReaderSetParserProp( reader, XML_PARSER_VALIDATE, 1 );     
+		ret = xmlTextReaderRead(reader);
+        while (ret == 1) 
+		{
+			selectReader(reader);
+            ret = xmlTextReaderRead(reader);
+        }
+
+		// Once the document has been fully parsed, check the validation results
+		if (xmlTextReaderIsValid(reader) != 1)
+		{
+			wxMessageBox(_("Model is not following embedded DTD definition, check it."), _("Error"), wxICON_ERROR);
+			return false;
+		}
+
+		if (ret != 0) {
+            wxMessageBox(_("Failed to load model file"), _("Error"), wxICON_ERROR);
+			return false;
+		}
+		else
+		{
+			return true;
+		}
+	}
+	return false;
+}
+
+wxString ddXmlStorage::getNodeName(xmlTextReaderPtr reader)
+{
+	xmlChar *name;
+	wxString out = wxEmptyString;
+	name = xmlTextReaderName(reader);
+    if (name == NULL)
+        name = xmlStrdup(BAD_CAST "--");
+	out = WXSTRING_FROM_XML(name);
+	xmlFree(name);
+	return out;
+}
+
+int ddXmlStorage::getNodeType(xmlTextReaderPtr reader)
+{
+	return xmlTextReaderNodeType(reader);
+}
+
+wxString ddXmlStorage::getNodeValue(xmlTextReaderPtr reader)
+{
+	wxString out = wxEmptyString;
+	xmlChar *value;	
+	value = xmlTextReaderValue(reader);
+	if (value != NULL)
+	{
+		out=WXSTRING_FROM_XML(value);
+		xmlFree(value);
+	}
+	return out;
+}
+
+
+void ddXmlStorage::selectReader(xmlTextReaderPtr reader)
+{
+	if(getNodeType(reader)==1) //libxml 1 for start element
+	{
+		if(getNodeName(reader).IsSameAs(_("MODEL"),false))
+		{
+			//<!ELEMENT MODEL (TABLE+)>
+		}
+
+		if(getNodeName(reader).IsSameAs(_("TABLE"),false))
+		{
+			getTable(reader);
+			design->refreshDraw();
+		}
+
+		if(getNodeName(reader).IsSameAs(_("RELATIONSHIP"),false))
+		{
+			ddRelationshipFigure *r = getRelationship(reader);
+			design->addTable(r);
+			design->refreshDraw();
+		}
+
+	}
+}
+
+// Table isn't created here because fk needs all tables created before assign it
+ddTableFigure* ddXmlStorage::getTable(xmlTextReaderPtr reader)
+{
+/*
+<!ELEMENT TABLE (Attribute*,POINT,TITLE,UKNAMES?, PKNAME, COLUMNS, BEGINDRAWCOLS, BEGINDRAWIDXS, MAXCOLINDEX, MINIDXINDEX, MAXIDXINDEX, COLSROWSSIZE, COLSWINDOW,  IDXSROWSSIZE, IDXSWINDOW )>
+<!ELEMENT COLUMNS (COLUMN*)>
+<!ELEMENT UKNAMES (UKNAME+)>
+<!ELEMENT UKNAME (#PCDATA)>
+<!ELEMENT PKNAME (#PCDATA)>
+<!ELEMENT BEGINDRAWCOLS (#PCDATA)>
+<!ELEMENT BEGINDRAWIDXS (#PCDATA)>
+<!ELEMENT MAXCOLINDEX (#PCDATA)>
+<!ELEMENT MINIDXINDEX (#PCDATA)>
+<!ELEMENT MAXIDXINDEX (#PCDATA)>
+<!ELEMENT COLSROWSSIZE (#PCDATA)>
+<!ELEMENT COLSWINDOW (#PCDATA)>
+<!ELEMENT IDXSROWSSIZE (#PCDATA)>
+<!ELEMENT IDXSWINDOW (#PCDATA)>
+
+<!ATTLIST TABLE	TableID ID #REQUIRED >
+*/
+
+int tmp;
+wxString TableID,node,tmpInt;
+xmlChar *value;
+
+	//<!ATTLIST TABLE	TableID ID #REQUIRED >
+	tmp = xmlTextReaderHasAttributes(reader);
+	TableID=_("TableID");
+	if(tmp)
+	{
+		value = xmlTextReaderGetAttribute(reader,XML_FROM_WXSTRING(TableID));
+	}
+	
+	if(value)
+	{
+			TableID = WXSTRING_FROM_XML(value);
+			xmlFree(value);
+	}
+
+	// --> ATTRIBUTE*
+	//Element(s) Attribute*
+	// NEED TO IMPLEMENT THIS
+
+	// --> POINT
+	//<!ELEMENT POINT (X,Y)>
+	int x=0,y=0;
+	tmp = xmlTextReaderRead(reader);	//go POINT
+	tmp = xmlTextReaderRead(reader);	//go X
+	tmp = xmlTextReaderRead(reader);	//go X Value
+	value = xmlTextReaderValue(reader);  //Value of X
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		x = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /X
+
+	tmp = xmlTextReaderRead(reader);	//go Y
+	tmp = xmlTextReaderRead(reader);	//go Y Value
+	value = xmlTextReaderValue(reader);  //Value of Y
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		y = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /Y
+	tmp = xmlTextReaderRead(reader);	//go /POINT
+
+	// --> TITLE
+	//<!ELEMENT TITLE (NAME,ALIAS?)>
+	//<!ELEMENT NAME (#PCDATA)>
+	//<!ELEMENT ALIAS (#PCDATA)>
+	wxString tableName = wxEmptyString;
+	wxString tableAlias = wxEmptyString;
+
+	tmp = xmlTextReaderRead(reader);	//go to TITLE
+	tmp = xmlTextReaderRead(reader);	//go to NAME
+	tmp = xmlTextReaderRead(reader);	//go to NAME Value
+	value = xmlTextReaderValue(reader);  //Value of NAME
+
+	if(value)
+	{
+		tableName = WXSTRING_FROM_XML(value);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /NAME
+	tmp = xmlTextReaderRead(reader);	//go to ALIAS or /TITLE
+	if(getNodeName(reader).IsSameAs(_("ALIAS"),false))
+	{
+		tmp = xmlTextReaderRead(reader);	//go to ALIAS Value	
+		value = xmlTextReaderValue(reader);  //Value of ALIAS
+		if(value)
+		{
+			tableAlias = WXSTRING_FROM_XML(value);
+			xmlFree(value);
+		}
+		tmp = xmlTextReaderRead(reader);	//go to /ALIAS
+		tmp = xmlTextReaderRead(reader);	//go to /TITLE
+	}
+	
+	// --> UKNAMES?
+	//<!ELEMENT UKNAMES (UKNAME+)>
+	//<!ELEMENT UKNAME (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to UKNAMES or PKNAMES?
+	wxArrayString ukNames;
+	if(getNodeName(reader).IsSameAs(_("UKNAMES"),false))
+	{
+			tmp = xmlTextReaderRead(reader);	//go UKNAME
+		do{
+			tmp = xmlTextReaderRead(reader);	//go UKNAME Value
+			value = xmlTextReaderValue(reader);  //Value of UKNAME
+			if(value)
+			{
+				ukNames.Add(WXSTRING_FROM_XML(value));
+				xmlFree(value);
+			}
+			tmp = xmlTextReaderRead(reader);	//go to /UKNAME
+			tmp = xmlTextReaderRead(reader);	//go to UKNAME or /UKNAMES ?
+		}while(getNodeName(reader).IsSameAs(_("UKNAME"),false));
+		tmp = xmlTextReaderRead(reader);	//go to PKNAME
+	}
+
+	// --> PKNAME
+	//<!ELEMENT PKNAME (#PCDATA)>
+	wxString pkName;
+	if(getNodeName(reader).IsSameAs(_("PKNAME")))
+	{
+		tmp = xmlTextReaderRead(reader);	//go to PKNAME Value
+		value = xmlTextReaderValue(reader);  //Value of PKNAME
+			if(value)
+			{
+				pkName = WXSTRING_FROM_XML(value);
+				xmlFree(value);
+			}
+		tmp = xmlTextReaderRead(reader);	//go to /PKNAME
+	}
+	
+	int beginDrawCols, beginDrawIdxs, maxColIndex, minIdxIndex, maxIdxIndex, colsRowsSize, colsWindow, idxsRowsSize, idxsWindow;
+	// --> BEGINDRAWCOLS
+	//<!ELEMENT BEGINDRAWCOLS (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to BEGINDRAWCOLS
+	tmp = xmlTextReaderRead(reader);	//go to BEGINDRAWCOLS Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		beginDrawCols = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /BEGINDRAWCOLS	
+	tmp = xmlTextReaderRead(reader);	//go to BEGINDRAWIDXS	
+
+	// --> BEGINDRAWIDXS
+	//<!ELEMENT BEGINDRAWIDXS (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to BEGINDRAWIDXS Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		beginDrawIdxs = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /BEGINDRAWIDXS	
+	tmp = xmlTextReaderRead(reader);	//go to MAXCOLINDEX
+
+	// --> MAXCOLINDEX
+	//<!ELEMENT MAXCOLINDEX (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to MAXCOLINDEX Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		maxColIndex = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /MAXCOLINDEX	
+	tmp = xmlTextReaderRead(reader);	//go to MINIDXINDEX
+	
+	// --> MINIDXINDEX
+	//<!ELEMENT MINIDXINDEX (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to MINIDXINDEX Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		minIdxIndex = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /MINIDXINDEX	
+	tmp = xmlTextReaderRead(reader);	//go to MAXIDXINDEX
+	
+	// --> MAXIDXINDEX
+	//<!ELEMENT MAXIDXINDEX (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to MAXIDXINDEX Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		maxIdxIndex = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /MAXIDXINDEX	
+	tmp = xmlTextReaderRead(reader);	//go to COLSROWSSIZE
+	
+	// --> COLSROWSSIZE
+	//<!ELEMENT COLSROWSSIZE (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to COLSROWSSIZE Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		colsRowsSize = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /COLSROWSSIZE	
+	tmp = xmlTextReaderRead(reader);	//go to COLSWINDOW
+	
+	// --> COLSWINDOW
+	//<!ELEMENT COLSWINDOW (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to COLSWINDOW Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		colsWindow = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /COLSWINDOW	
+	tmp = xmlTextReaderRead(reader);	//go to IDXSROWSSIZE
+	
+	// --> IDXSROWSSIZE
+	//<!ELEMENT IDXSROWSSIZE (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to IDXSROWSSIZE Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		idxsRowsSize = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /IDXSROWSSIZE	
+	tmp = xmlTextReaderRead(reader);	//go to IDXSWINDOW
+	
+	// --> IDXSWINDOW
+	//<!ELEMENT IDXSWINDOW (#PCDATA)>*/
+	tmp = xmlTextReaderRead(reader);	//go to IDXSWINDOW Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		idxsWindow = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /IDXSWINDOW	
+	// --> COLUMNS
+	//<!ELEMENT COLUMNS (COLUMN*)>
+
+	tmp = xmlTextReaderRead(reader);	//go to COLUMNS
+	
+	// Use empty table without columns created at preparsing to fill it with metadata
+	ddTableFigure *t = design->getTable(tableName);
+	if(t!=NULL)
+		t->InitTableValues(ukNames, pkName, beginDrawCols, beginDrawIdxs, maxColIndex, minIdxIndex, maxIdxIndex, colsRowsSize, colsWindow, idxsRowsSize, idxsWindow);
+	else
+		wxMessageBox(_("Table initial metadata not found"), _("Error"), wxICON_ERROR);
+
+	//COLUMNS node have COLUMN children?
+	if(getNodeName(reader).IsSameAs(_("COLUMNS"))&& getNodeType(reader) == 1 && !xmlTextReaderIsEmptyElement(reader) )	
+	{
+		ddColumnFigure *c;
+		do{
+			c = getColumn(reader, t); 
+			if(c)
+			{
+				t->addColumnFromStorage(c);
+				c->setRightIconForColumn();
+			}
+		}while(c!=NULL);
+		//now at </COLUMNS>
+	} 
+
+	tmp = xmlTextReaderRead(reader);	//go to </TABLE>
+	t->syncPositionsAfterLoad();  //synchronize positions
+	t->updateTableSize();
+	return t;
+}
+
+ddColumnFigure* ddXmlStorage::getColumn(xmlTextReaderPtr reader, ddTableFigure *colOwner)
+{
+/*
+<!ELEMENT COLUMN (Attribute*,NAME,OPTION,UKINDEX,ISPK,COLUMNTYPE,PRECISION?,SCALE?,GENERATEFKNAME)>
+<!ELEMENT NAME (#PCDATA)>
+<!ELEMENT OPTION (#PCDATA)>
+<!ELEMENT UKINDEX (#PCDATA)>
+<!ELEMENT ISPK (#PCDATA)>
+<!ELEMENT COLUMNTYPE (#PCDATA)>
+<!ELEMENT PRECISION (#PCDATA)>
+<!ELEMENT SCALE (#PCDATA)>
+<!ELEMENT GENERATEFKNAME (#PCDATA)>
+*/
+
+	xmlChar *value;	
+	int tmp;
+	tmp = xmlTextReaderRead(reader);	//go to COLUMN
+	if(getNodeName(reader).IsSameAs(_("COLUMN"))&& getNodeType(reader) == 1 && !xmlTextReaderIsEmptyElement(reader) )
+	{
+		// --> ATTRIBUTE*
+		//Element(s) Attribute*
+		// NEED TO IMPLEMENT THIS
+
+		// --> NAME
+		//<!ELEMENT NAME (#PCDATA)>
+		wxString columnName;
+		tmp = xmlTextReaderRead(reader);	//go to NAME
+		tmp = xmlTextReaderRead(reader);	//go to NAME Value
+		value = xmlTextReaderValue(reader);  //Value of NAME
+			if(value)
+			{
+				columnName = WXSTRING_FROM_XML(value);
+				xmlFree(value);
+			}
+		tmp = xmlTextReaderRead(reader);	//go to /NAME
+		tmp = xmlTextReaderRead(reader);	//go to OPTION
+
+		// --> OPTION
+		//<!ELEMENT OPTION (#PCDATA)>
+		int option;
+		wxString tmpInt;
+		tmp = xmlTextReaderRead(reader);	//go to OPTION Value	
+		value = xmlTextReaderValue(reader);  //Value
+		if(value)
+		{
+			tmpInt = WXSTRING_FROM_XML(value);
+			option = wxAtoi(tmpInt);
+			xmlFree(value);
+		}
+		tmp = xmlTextReaderRead(reader);	//go to /OPTION	
+		tmp = xmlTextReaderRead(reader);	//go to UKINDEX
+
+		// --> UKINDEX
+		//<!ELEMENT UKINDEX (#PCDATA)>
+		int ukindex;
+		tmp = xmlTextReaderRead(reader);	//go to UKINDEX Value	
+		value = xmlTextReaderValue(reader);  //Value
+		if(value)
+		{
+			tmpInt = WXSTRING_FROM_XML(value);
+			ukindex = wxAtoi(tmpInt);
+			xmlFree(value);
+		}
+		tmp = xmlTextReaderRead(reader);	//go to /UKINDEX	
+		tmp = xmlTextReaderRead(reader);	//go to ISPK
+
+		// --> ISPK
+		//<!ELEMENT ISPK (#PCDATA)>
+		bool isPk=false;
+		tmp = xmlTextReaderRead(reader);	//go to ISPK Value
+		value = xmlTextReaderValue(reader);  //Value of ISPK
+			if(value)
+			{
+				tmpInt = WXSTRING_FROM_XML(value);
+				isPk = tmpInt.IsSameAs(_("T"));
+				xmlFree(value);
+			}
+		tmp = xmlTextReaderRead(reader);	//go to /ISPK
+
+		// --> COLUMNTYPE
+		//<!ELEMENT COLUMNTYPE (#PCDATA)>
+		wxString columnType;
+		tmp = xmlTextReaderRead(reader);	//go to COLUMNTYPE
+		tmp = xmlTextReaderRead(reader);	//go to COLUMNTYPE Value
+		value = xmlTextReaderValue(reader);  //Value of COLUMNTYPE
+			if(value)
+			{
+				columnType = WXSTRING_FROM_XML(value);
+				xmlFree(value);
+			}
+		tmp = xmlTextReaderRead(reader);	//go to /COLUMNTYPE
+
+		// --> PRECISION?
+		//<!ELEMENT PRECISION (#PCDATA)>
+		tmp = xmlTextReaderRead(reader);	//go to PRECISION? or SCALE? or ALIAS? or GENERATEFKNAME
+		int precision = -1;
+		if(getNodeName(reader).IsSameAs(_("PRECISION")))
+		{
+			tmp = xmlTextReaderRead(reader);	//go to PRECISION Value
+			value = xmlTextReaderValue(reader);  //Value of PRECISION
+				if(value)
+				{
+					tmpInt = WXSTRING_FROM_XML(value);
+					precision = wxAtoi(tmpInt);
+					xmlFree(value);
+				}
+			tmp = xmlTextReaderRead(reader);	//go to /PRECISION
+			tmp = xmlTextReaderRead(reader);	//go to SCALE? or ALIAS? or GENERATEFKNAME
+		}
+
+		// --> 	SCALE?
+		//<!ELEMENT SCALE (#PCDATA)>
+
+		int scale = -1;
+		if(getNodeName(reader).IsSameAs(_("SCALE")))
+		{
+			tmp = xmlTextReaderRead(reader);	//go to SCALE Value
+			value = xmlTextReaderValue(reader);  //Value of SCALE
+				if(value)
+				{
+					tmpInt = WXSTRING_FROM_XML(value);
+					scale = wxAtoi(tmpInt);
+					xmlFree(value);
+				}
+			tmp = xmlTextReaderRead(reader);	//go to /SCALE
+			tmp = xmlTextReaderRead(reader);	//go to ALIAS? or GENERATEFKNAME
+		}
+
+		// --> GENERATEFKNAME
+		//<!ELEMENT GENERATEFKNAME (#PCDATA)>
+		bool generateFkName=false;
+		tmp = xmlTextReaderRead(reader);	//go to GENERATEFKNAME Value
+		value = xmlTextReaderValue(reader);  //Value of GENERATEFKNAME
+			if(value)
+			{
+				tmpInt = WXSTRING_FROM_XML(value);
+				generateFkName = tmpInt.IsSameAs(_("T"));
+				xmlFree(value);
+			}
+		tmp = xmlTextReaderRead(reader);	//go to /GENERATEFKNAME
+		tmp = xmlTextReaderRead(reader);	//go to /COLUMN
+		
+		return new ddColumnFigure(columnName,colOwner,(ddColumnOptionType)option,generateFkName,isPk,columnType,precision,scale,ukindex,NULL,NULL);
+	}
+	return NULL;   //found /COLUMNS node
+}
+
+ddRelationshipFigure* ddXmlStorage::getRelationship(xmlTextReaderPtr reader)
+{
+/*
+<!-- Points Element -->
+<!ELEMENT POINTS (POINT, POINT,POINT*)>
+<!ELEMENT POINT (X,Y)>
+<!ELEMENT X (#PCDATA)>
+<!ELEMENT Y (#PCDATA)>
+
+<!-- Relationship Element -->
+<!ELEMENT RELATIONSHIP (Attribute*, POINTS, RELATIONITEMS, UKINDEX, NAME, ONUPDATE, ONDELETE, MATCHSIMPLE, IDENTIFYING, ONETOMANY, MANDATORY, FKFROMPK)>
+<!ELEMENT RELATIONITEMS (RELATIONITEM*)>
+<!ELEMENT ONUPDATE (#PCDATA)>
+<!ELEMENT ONDELETE (#PCDATA)>
+<!ELEMENT MATCHSIMPLE (#PCDATA)>
+<!ELEMENT IDENTIFYING (#PCDATA)>
+<!ELEMENT ONETOMANY (#PCDATA)>
+<!ELEMENT MANDATORY (#PCDATA)>
+<!ELEMENT FKFROMPK (#PCDATA)>
+
+<!ATTLIST RELATIONSHIP	SourceTableID IDREF #REQUIRED >
+<!ATTLIST RELATIONSHIP	DestTableID IDREF #REQUIRED >	
+*/
+
+	xmlChar *value;	
+	wxString tmpInt;
+	int tmp;
+
+	//<!ATTLIST RELATIONSHIP SourceTableID IDREF #REQUIRED >
+	tmp = xmlTextReaderHasAttributes(reader);
+	wxString SourceTableID=_("SourceTableID");
+	if(tmp)
+	{
+		value = xmlTextReaderGetAttribute(reader,XML_FROM_WXSTRING(SourceTableID));
+	}
+	
+	if(value)
+	{
+			SourceTableID = WXSTRING_FROM_XML(value);
+			xmlFree(value);
+	}
+	
+	//<!ATTLIST RELATIONSHIP DestTableID IDREF #REQUIRED >	
+	tmp = xmlTextReaderHasAttributes(reader);
+	wxString DestTableID=_("DestTableID");
+	if(tmp)
+	{
+		value = xmlTextReaderGetAttribute(reader,XML_FROM_WXSTRING(DestTableID));
+	}
+	
+	if(value)
+	{
+			DestTableID = WXSTRING_FROM_XML(value);
+			xmlFree(value);
+	}
+	
+	ddTableFigure *source = design->getTable(design->getTableName(SourceTableID));
+	ddTableFigure *destination = design->getTable(design->getTableName(DestTableID));
+
+	ddRelationshipFigure *relation = new ddRelationshipFigure();
+	relation->setStartTerminal(new ddRelationshipTerminal(relation, false));
+	relation->setEndTerminal(new ddRelationshipTerminal(relation, true));
+	relation->clearPoints();
+
+	// --> ATTRIBUTE*
+	//Element(s) Attribute*
+	// NEED TO IMPLEMENT THIS
+
+	// --> POINTS
+	//<!ELEMENT POINTS (POINT, POINT,POINT*)>
+	int x,y;
+
+	tmp = xmlTextReaderRead(reader);	//go to POINTS
+	if(getNodeName(reader).IsSameAs(_("POINTS"),false))
+	{
+			tmp = xmlTextReaderRead(reader);	//go POINT
+		do{
+			tmp = xmlTextReaderRead(reader);	//go X
+			tmp = xmlTextReaderRead(reader);	//go X Value
+			value = xmlTextReaderValue(reader);  //Value of X
+			if(value)
+			{
+				tmpInt = WXSTRING_FROM_XML(value);
+				x = wxAtoi(tmpInt);
+				xmlFree(value);
+			}
+			tmp = xmlTextReaderRead(reader);	//go to /X
+
+			tmp = xmlTextReaderRead(reader);	//go Y
+			tmp = xmlTextReaderRead(reader);	//go Y Value
+			value = xmlTextReaderValue(reader);  //Value of Y
+			if(value)
+			{
+				tmpInt = WXSTRING_FROM_XML(value);
+				y = wxAtoi(tmpInt);
+				xmlFree(value);
+			}
+			tmp = xmlTextReaderRead(reader);	//go to /Y
+			tmp = xmlTextReaderRead(reader);	//go /POINT
+			
+			relation->addPoint(x,y);
+			
+			tmp = xmlTextReaderRead(reader);	//go to POINT or /POINTS ?
+		}while(getNodeName(reader).IsSameAs(_("POINT"),false));
+	}
+	
+	// --> UKINDEX
+	//<!ELEMENT UKINDEX (#PCDATA)>
+	int ukindex;
+	tmp = xmlTextReaderRead(reader);	//go to UKINDEX
+	tmp = xmlTextReaderRead(reader);	//go to UKINDEX Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		ukindex = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /UKINDEX	
+	
+	// --> NAME
+	//<!ELEMENT NAME (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to NAME or ONUPDATE
+	wxString RelationshipName=wxEmptyString;
+	if(getNodeName(reader).IsSameAs(_("NAME"),false))
+	{
+
+		tmp = xmlTextReaderRead(reader);	//go to NAME Value
+		value = xmlTextReaderValue(reader);  //Value of NAME
+		if(value)
+		{
+			RelationshipName = WXSTRING_FROM_XML(value);
+			xmlFree(value);
+		}
+		tmp = xmlTextReaderRead(reader);	//go to /NAME
+		tmp = xmlTextReaderRead(reader);	//go to ONUPDATE
+	}
+
+	// --> ONUPDATE
+	//<!ELEMENT ONUPDATE (#PCDATA)>
+	int onUpdate;
+
+	tmp = xmlTextReaderRead(reader);	//go to ONUPDATE Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		onUpdate = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /ONUPDATE	
+	tmp = xmlTextReaderRead(reader);	//go to ONDELETE
+
+	// --> ONDELETE
+	//<!ELEMENT ONDELETE (#PCDATA)>
+	int onDelete;
+	tmp = xmlTextReaderRead(reader);	//go to ONDELETE Value	
+	value = xmlTextReaderValue(reader);  //Value
+	if(value)
+	{
+		tmpInt = WXSTRING_FROM_XML(value);
+		onDelete = wxAtoi(tmpInt);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /ONDELETE	
+	tmp = xmlTextReaderRead(reader);	//go to MATCHSIMPLE
+
+	// --> MATCHSIMPLE
+	//<!ELEMENT MATCHSIMPLE (#PCDATA)>
+	bool matchSimple=false;
+	tmp = xmlTextReaderRead(reader);	//go to MATCHSIMPLE Value
+	value = xmlTextReaderValue(reader);  //Value of MATCHSIMPLE
+		if(value)
+		{
+			tmpInt = WXSTRING_FROM_XML(value);
+			matchSimple = tmpInt.IsSameAs(_("T"));
+			xmlFree(value);
+		}
+	tmp = xmlTextReaderRead(reader);	//go to /MATCHSIMPLE
+	tmp = xmlTextReaderRead(reader);	//go to IDENTIFYING
+
+	// --> IDENTIFYING
+	//<!ELEMENT IDENTIFYING (#PCDATA)>
+	bool identifying=false;
+	tmp = xmlTextReaderRead(reader);	//go to IDENTIFYING Value
+	value = xmlTextReaderValue(reader);  //Value of IDENTIFYING
+		if(value)
+		{
+			tmpInt = WXSTRING_FROM_XML(value);
+			identifying = tmpInt.IsSameAs(_("T"));
+			xmlFree(value);
+		}
+	tmp = xmlTextReaderRead(reader);	//go to /IDENTIFYING
+	tmp = xmlTextReaderRead(reader);	//go to ONETOMANY		
+
+	// --> ONETOMANY
+	//<!ELEMENT ONETOMANY (#PCDATA)>
+	bool oneToMany=false;
+	tmp = xmlTextReaderRead(reader);	//go to ONETOMANY Value
+	value = xmlTextReaderValue(reader);  //Value of ONETOMANY
+		if(value)
+		{
+			tmpInt = WXSTRING_FROM_XML(value);
+			oneToMany = tmpInt.IsSameAs(_("T"));
+			xmlFree(value);
+		}
+	tmp = xmlTextReaderRead(reader);	//go to /ONETOMANY
+	tmp = xmlTextReaderRead(reader);	//go to MANDATORY	
+
+	// --> MANDATORY
+	//<!ELEMENT MANDATORY (#PCDATA)>
+	bool mandatory=false;
+	tmp = xmlTextReaderRead(reader);	//go to MANDATORY Value
+	value = xmlTextReaderValue(reader);  //Value of MANDATORY
+		if(value)
+		{
+			tmpInt = WXSTRING_FROM_XML(value);
+			mandatory = tmpInt.IsSameAs(_("T"));
+			xmlFree(value);
+		}
+	tmp = xmlTextReaderRead(reader);	//go to /MANDATORY
+	tmp = xmlTextReaderRead(reader);	//go to FKFROMPK	
+
+	// --> FKFROMPK
+	//<!ELEMENT FKFROMPK (#PCDATA)>
+	bool fkFromPk=false;
+	tmp = xmlTextReaderRead(reader);	//go to FKFROMPK Value
+	value = xmlTextReaderValue(reader);  //Value of FKFROMPK
+		if(value)
+		{
+			tmpInt = WXSTRING_FROM_XML(value);
+			fkFromPk = tmpInt.IsSameAs(_("T"));
+			xmlFree(value);
+		}
+	tmp = xmlTextReaderRead(reader);	//go to /FKFROMPK
+	
+
+	//refresh relationship after all values were setted
+	relation->initRelationValues(source,destination,ukindex, RelationshipName, (actionKind) onUpdate, (actionKind) onDelete,matchSimple,identifying,oneToMany,mandatory,fkFromPk);
+
+
+	// --> RELATIONITEMS
+	//<!ELEMENT RELATIONITEMS (RELATIONITEM*)>
+	tmp = xmlTextReaderRead(reader);	//go to RELATIONITEMS
+	if(getNodeName(reader).IsSameAs(_("RELATIONITEMS"))&& getNodeType(reader) == 1 && !xmlTextReaderIsEmptyElement(reader) )	
+	{
+			tmp = xmlTextReaderRead(reader);	//go RELATIONITEM
+		do{
+			//Now only add item to relationship
+			ddRelationshipItem *item = getRelationshipItem(reader,relation,source,destination);
+			relation->getItemsHashMap()[item->original->getColumnName()]=item;
+
+			tmp = xmlTextReaderRead(reader);	//go to RELATIONITEM or /RELATIONITEMS
+		}while(getNodeName(reader).IsSameAs(_("RELATIONITEM"),false));
+	}
+
+	tmp = xmlTextReaderRead(reader);	//go to /RELATIONSHIP	
+
+	relation->updateConnection();
+	return relation;
+}
+
+ddRelationshipItem* ddXmlStorage::getRelationshipItem(xmlTextReaderPtr reader, ddRelationshipFigure *itemOwner, ddTableFigure *source, ddTableFigure *destination)
+{
+/*
+<!ELEMENT RELATIONITEM (AUTOGENFK, FKCOLNAME,SOURCECOLNAME, INITIALCOLNAME, INITIALALIASNAME )>
+<!ELEMENT AUTOGENFK (#PCDATA)>
+<!ELEMENT FKCOLNAME (#PCDATA)>
+<!ELEMENT SOURCECOLNAME (#PCDATA)>
+<!ELEMENT INITIALCOLNAME (#PCDATA)>
+<!ELEMENT INITIALALIASNAME (#PCDATA)>
+*/
+	xmlChar *value;	
+	wxString tmpInt;
+	int tmp;
+	
+	// --> AUTOGENFK
+	//<!ELEMENT AUTOGENFK (#PCDATA)>
+	bool autoGenFk=false;
+	tmp = xmlTextReaderRead(reader);	//go to AUTOGENFK
+	tmp = xmlTextReaderRead(reader);	//go to AUTOGENFK Value
+	value = xmlTextReaderValue(reader);  //Value of AUTOGENFK
+		if(value)
+		{
+			tmpInt = WXSTRING_FROM_XML(value);
+			autoGenFk = tmpInt.IsSameAs(_("T"));
+			xmlFree(value);
+		}
+	tmp = xmlTextReaderRead(reader);	//go to /AUTOGENFK
+
+	//<!ELEMENT FKCOLNAME (#PCDATA)>
+	wxString fkColName;
+	tmp = xmlTextReaderRead(reader);	//go to FKCOLNAME
+	tmp = xmlTextReaderRead(reader);	//go to FKCOLNAME Value
+	value = xmlTextReaderValue(reader);  //Value of FKCOLNAME
+	if(value)
+	{
+		fkColName = WXSTRING_FROM_XML(value);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /FKCOLNAME
+
+	//<!ELEMENT SOURCECOLNAME (#PCDATA)>
+	wxString sourceColName;
+	tmp = xmlTextReaderRead(reader);	//go to SOURCECOLNAME
+	tmp = xmlTextReaderRead(reader);	//go to SOURCECOLNAME Value
+	value = xmlTextReaderValue(reader);  //Value of SOURCECOLNAME
+	if(value)
+	{
+		sourceColName = WXSTRING_FROM_XML(value);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /SOURCECOLNAME
+
+	//<!ELEMENT INITIALCOLNAME (#PCDATA)>
+	wxString initialColName;
+	tmp = xmlTextReaderRead(reader);	//go to INITIALCOLNAME
+	tmp = xmlTextReaderRead(reader);	//go to INITIALCOLNAME Value
+	value = xmlTextReaderValue(reader);  //Value of INITIALCOLNAME
+	if(value)
+	{
+		initialColName = WXSTRING_FROM_XML(value);
+		xmlFree(value);
+	}
+	tmp = xmlTextReaderRead(reader);	//go to /INITIALCOLNAME
+
+	//<!ELEMENT INITIALALIASNAME (#PCDATA)>
+	tmp = xmlTextReaderRead(reader);	//go to INITIALALIASNAME
+	wxString initialAliasName=wxEmptyString;
+	if(getNodeName(reader).IsSameAs(_("INITIALALIASNAME"),false))
+	{
+		tmp = xmlTextReaderRead(reader);	//go to INITIALALIASNAME Value
+		value = xmlTextReaderValue(reader);  //Value of INITIALALIASNAME
+		if(value)
+		{
+			initialAliasName = WXSTRING_FROM_XML(value);
+			xmlFree(value);
+		}
+		tmp = xmlTextReaderRead(reader);	//go to /INITIALALIASNAME
+		tmp = xmlTextReaderRead(reader);	//go to /RELATIONITEM
+	}
+
+	ddRelationshipItem *item = new ddRelationshipItem();
+	ddColumnFigure *sourceCol = source->getColByName(sourceColName);
+	ddColumnFigure *destinationCol = destination->getColByName(fkColName);
+	item->initRelationshipItemValues(itemOwner, destination, autoGenFk, destinationCol, sourceCol, initialColName, initialAliasName);
+
+	return item;
+
+}
+
+//Temporary DTD will change later
+wxString ddXmlStorage::getModelDTD()
+{
+	wxString dtd = wxEmptyString;
+dtd += _("<!ELEMENT MODEL (TABLE+,RELATIONSHIP*)>");
+dtd += _("<!--Atribute Element-->");
+dtd += _("<!ELEMENT Attribute (#PCDATA)>");
+dtd += _(" ");
+dtd += _("<!ATTLIST Attribute");
+dtd += _("	Name CDATA #REQUIRED");
+dtd += _("	Kind (normal | selected) #REQUIRED");
+dtd += _(">");
+dtd += _("<!--Rectangle Element-->");
+dtd += _("<!ELEMENT Rect (x,y,w,h)>");
+dtd += _("<!ATTLIST Name");
+dtd += _("	Name CDATA #REQUIRED");
+dtd += _(">");
+dtd += _("<!ELEMENT 	x (#PCDATA)>");
+dtd += _("<!ELEMENT 	y (#PCDATA)>");
+dtd += _("<!ELEMENT 	w (#PCDATA)>");
+dtd += _("<!ELEMENT 	h (#PCDATA)>");
+dtd += _(" ");
+dtd += _("<!--Column Element-->");
+dtd += _("<!ELEMENT COLUMN (Attribute*,NAME,OPTION,UKINDEX,ISPK,COLUMNTYPE,PRECISION?,SCALE?,GENERATEFKNAME)>");
+dtd += _("<!ELEMENT NAME (#PCDATA)>");
+dtd += _("<!ELEMENT OPTION (#PCDATA)>");
+dtd += _("<!ELEMENT UKINDEX (#PCDATA)>");
+dtd += _("<!ELEMENT ISPK (#PCDATA)>");
+dtd += _("<!ELEMENT COLUMNTYPE (#PCDATA)>");
+dtd += _("<!ELEMENT PRECISION (#PCDATA)>");
+dtd += _("<!ELEMENT SCALE (#PCDATA)>");
+dtd += _("<!ELEMENT GENERATEFKNAME (#PCDATA)>");
+dtd += _(" ");
+dtd += _(" ");
+dtd += _("<!ELEMENT TITLE (NAME,ALIAS?)>");
+dtd += _("<!ELEMENT ALIAS (#PCDATA)>");
+dtd += _(" ");
+dtd += _("<!-- Table Element -->");
+dtd += _("<!ELEMENT TABLE (Attribute*,POINT,TITLE,UKNAMES?, PKNAME, BEGINDRAWCOLS, BEGINDRAWIDXS, MAXCOLINDEX, MINIDXINDEX, MAXIDXINDEX, COLSROWSSIZE,");
+dtd += _("COLSWINDOW,IDXSROWSSIZE, IDXSWINDOW, COLUMNS )>");
+dtd += _("<!ELEMENT COLUMNS (COLUMN*)>");
+dtd += _("<!ELEMENT UKNAMES (UKNAME+)>");
+dtd += _("<!ELEMENT UKNAME (#PCDATA)>");
+dtd += _("<!ELEMENT PKNAME (#PCDATA)>");
+dtd += _("<!ELEMENT BEGINDRAWCOLS (#PCDATA)>");
+dtd += _("<!ELEMENT BEGINDRAWIDXS (#PCDATA)>");
+dtd += _("<!ELEMENT MAXCOLINDEX (#PCDATA)>");
+dtd += _("<!ELEMENT MINIDXINDEX (#PCDATA)>");
+dtd += _("<!ELEMENT MAXIDXINDEX (#PCDATA)>");
+dtd += _("<!ELEMENT COLSROWSSIZE (#PCDATA)>");
+dtd += _("<!ELEMENT COLSWINDOW (#PCDATA)>");
+dtd += _("<!ELEMENT IDXSROWSSIZE (#PCDATA)>");
+dtd += _("<!ELEMENT IDXSWINDOW (#PCDATA)>");
+dtd += _(" ");
+dtd += _("<!ATTLIST TABLE");
+dtd += _("	TableID ID #REQUIRED");
+dtd += _("	>");
+dtd += _(" ");
+dtd += _(" ");
+dtd += _("<!-- Relationship Element -->");
+dtd += _("<!ELEMENT RELATIONITEMS (RELATIONITEM*)>");
+dtd += _("<!ELEMENT RELATIONITEM (AUTOGENFK, FKCOLNAME,SOURCECOLNAME, INITIALCOLNAME, INITIALALIASNAME? )>");
+dtd += _("<!ELEMENT AUTOGENFK (#PCDATA)>");
+dtd += _("<!ELEMENT FKCOLNAME (#PCDATA)>");
+dtd += _("<!ELEMENT SOURCECOLNAME (#PCDATA)>");
+dtd += _("<!ELEMENT INITIALCOLNAME (#PCDATA)>");
+dtd += _("<!ELEMENT INITIALALIASNAME (#PCDATA)>");
+dtd += _(" ");
+dtd += _(" ");
+dtd += _("<!-- Points Element -->");
+dtd += _("<!ELEMENT POINTS (POINT, POINT, POINT*)>");
+dtd += _("<!ELEMENT POINT (X,Y)>");
+dtd += _("<!ELEMENT X (#PCDATA)>");
+dtd += _("<!ELEMENT Y (#PCDATA)>");
+dtd += _(" ");
+dtd += _(" ");
+dtd += _("<!ELEMENT RELATIONSHIP (Attribute*, POINTS, UKINDEX, NAME?, ONUPDATE, ONDELETE, MATCHSIMPLE, IDENTIFYING, ONETOMANY, MANDATORY, FKFROMPK, RELATIONITEMS)>");
+dtd += _("<!ELEMENT ONUPDATE (#PCDATA)>");
+dtd += _("<!ELEMENT ONDELETE (#PCDATA)>");
+dtd += _("<!ELEMENT MATCHSIMPLE (#PCDATA)>");
+dtd += _("<!ELEMENT IDENTIFYING (#PCDATA)>");
+dtd += _("<!ELEMENT ONETOMANY (#PCDATA)>");
+dtd += _("<!ELEMENT MANDATORY (#PCDATA)>");
+dtd += _("<!ELEMENT FKFROMPK (#PCDATA)>");
+dtd += _(" ");
+dtd += _("<!ATTLIST RELATIONSHIP SourceTableID IDREF #REQUIRED >");
+dtd += _("<!ATTLIST RELATIONSHIP DestTableID IDREF #REQUIRED >");
+	return dtd;
+}
diff --git a/pgadmin/dd/dditems/figures/xml/module.mk b/pgadmin/dd/dditems/figures/xml/module.mk
new file mode 100644
index 0000000..70b47df
--- /dev/null
+++ b/pgadmin/dd/dditems/figures/xml/module.mk
@@ -0,0 +1,16 @@
+#######################################################################
+#
+# pgAdmin III - PostgreSQL Tools
+# 
+# Copyright (C) 2002 - 2011, The pgAdmin Development Team
+# This software is released under the PostgreSQL Licence
+#
+# module.mk - pgadmin/dd/dditems/figures/xml/ Makefile fragment
+#
+#######################################################################
+
+pgadmin3_SOURCES += \
+	$(srcdir)/dd/dditems/figures/xml/ddXmlStorage.cpp
+
+EXTRA_DIST += \
+	$(srcdir)/dd/dditems/figures/xml/module.mk
diff --git a/pgadmin/dd/dditems/handles/ddAddColButtonHandle.cpp b/pgadmin/dd/dditems/handles/ddAddColButtonHandle.cpp
index 5e70a5d..8fb35b8 100644
--- a/pgadmin/dd/dditems/handles/ddAddColButtonHandle.cpp
+++ b/pgadmin/dd/dditems/handles/ddAddColButtonHandle.cpp
@@ -46,7 +46,7 @@ void ddAddColButtonHandle::invokeStart(wxhdMouseEvent &event, wxhdDrawingView *v
 		if (answer == wxID_OK)
 		{
 			wxString name = nameDialog.GetValue();
-			if(table->colNameAvailable(name))
+			if(table->getColByName(name)==NULL)
 				table->addColumn(new ddColumnFigure(name, table));
 			else
 			{
diff --git a/pgadmin/dd/ddmodel/ddDatabaseDesign.cpp b/pgadmin/dd/ddmodel/ddDatabaseDesign.cpp
index 8eee6cd..143c861 100644
--- a/pgadmin/dd/ddmodel/ddDatabaseDesign.cpp
+++ b/pgadmin/dd/ddmodel/ddDatabaseDesign.cpp
@@ -14,6 +14,10 @@
 // wxWindows headers
 #include <wx/wx.h>
 
+// libxml2 headers
+#include <libxml/xmlwriter.h>
+#include <libxml/xmlreader.h>
+
 // App headers
 #include "dd/ddmodel/ddDatabaseDesign.h"
 #include "dd/wxhotdraw/tools/wxhdSelectionTool.h"
@@ -21,6 +25,7 @@
 #include "dd/dditems/figures/ddRelationshipFigure.h"
 #include "dd/dditems/utilities/ddDataType.h"
 #include "dd/ddmodel/ddDrawingEditor.h"
+#include "dd/dditems/figures/xml/ddXmlStorage.h"
 
 
 ddDatabaseDesign::ddDatabaseDesign(wxWindow *parent)
@@ -175,9 +180,9 @@ ddTableFigure *ddDatabaseDesign::getSelectedTable()
 	return table;
 }
 
-bool ddDatabaseDesign::containsTable(wxString tableName)
+ddTableFigure* ddDatabaseDesign::getTable(wxString tableName)
 {
-	bool out = false;
+	ddTableFigure *out = NULL;
 	wxhdIteratorBase *iterator = draw->model()->figuresEnumerator();
 	wxhdIFigure *tmp;
 	ddTableFigure *table;
@@ -189,10 +194,147 @@ bool ddDatabaseDesign::containsTable(wxString tableName)
 			table = (ddTableFigure *)tmp;
 			if(table->getTableName().Contains(tableName))
 			{
-				out = true;
+				out = table;
 			}
 		}
 	}
 	delete iterator;
 	return out;
 }
+
+#define XML_FROM_WXSTRING(s) ((xmlChar *)(const char *)s.mb_str(wxConvUTF8))
+#define WXSTRING_FROM_XML(s) wxString((char *)s, wxConvUTF8)
+
+bool ddDatabaseDesign::writeXmlModel(wxString file)
+{
+	int rc;
+
+	xmlWriter = xmlNewTextWriterFilename(file.mb_str(wxConvUTF8),0);
+	if (xmlWriter == NULL) {
+		wxMessageBox(_("Failed to write the model file!"), _("Error"), wxICON_ERROR);
+		return false;
+	}
+
+	rc = xmlTextWriterStartDocument(xmlWriter, NULL, "UTF-8" , NULL);
+	if(rc < 0)
+	{
+		wxMessageBox(_("Failed to write the model file!"), _("Error"), wxICON_ERROR);
+		return false;
+	}
+
+	ddXmlStorage::StartModel(xmlWriter,this);
+	//initialize IDs of tables
+	mappingNameToId.clear();
+	wxhdIteratorBase *iterator = draw->model()->figuresEnumerator();
+	wxhdIFigure *tmp;
+	ddTableFigure *table;
+	int nextID=10;
+	
+	while(iterator->HasNext())
+	{
+		tmp = (wxhdIFigure *)iterator->Next();
+		if(tmp->getKindId() == DDTABLEFIGURE)
+		{
+			table = (ddTableFigure *)tmp;
+			mappingNameToId[table->getTableName()]=wxString::Format(wxT("TID%d"),nextID);
+			nextID+=10;
+		}
+	}
+	delete iterator;
+	
+
+	//Create table xml info
+	iterator = draw->model()->figuresEnumerator();
+	while(iterator->HasNext())
+	{
+		tmp = (wxhdIFigure *)iterator->Next();
+		if(tmp->getKindId() == DDTABLEFIGURE)
+		{
+			table = (ddTableFigure *)tmp;
+			ddXmlStorage::Write(xmlWriter,table);
+		}
+	}
+	delete iterator;
+
+
+	//Create relationships xml info
+	ddRelationshipFigure *relationship;
+	iterator = draw->model()->figuresEnumerator();
+	while(iterator->HasNext())
+	{
+		tmp = (wxhdIFigure *)iterator->Next();
+		if(tmp->getKindId() == DDRELATIONSHIPFIGURE)
+		{
+			relationship = (ddRelationshipFigure *)tmp;
+			ddXmlStorage::Write(xmlWriter,relationship);
+		}
+	}
+	delete iterator;
+
+	ddXmlStorage::EndModel(xmlWriter);
+	
+	rc = xmlTextWriterEndDocument(xmlWriter);
+	if(rc < 0)
+	{
+		wxMessageBox(_("Failed to write the model file!"), _("Error"), wxICON_ERROR);
+		return false;
+	}
+	
+	xmlFreeTextWriter(xmlWriter);
+	return true;
+}
+
+bool ddDatabaseDesign::readXmlModel(wxString file)
+{
+	eraseModel();
+	mappingIdToName.clear();
+	
+	//Initial Parse Model
+	xmlTextReaderPtr reader = xmlReaderForFile(file.mb_str(wxConvUTF8), NULL, 0);
+    if (!reader)
+    {
+        wxMessageBox(_("Failed to load model file!"), _("Error"), wxICON_ERROR);
+        return false;
+    }
+
+	ddXmlStorage::setModel(this);
+	ddXmlStorage::initialModelParse(reader);
+	
+	//Parse Model
+	xmlReaderNewFile(reader,file.mb_str(wxConvUTF8),NULL,0);
+	ddXmlStorage::setModel(this);
+	if(!ddXmlStorage::Read(reader))
+	{
+        wxMessageBox(_("Failed to load model file!"), _("Error"), wxICON_ERROR);
+		return false;
+	}
+	xmlFreeTextReader(reader);
+	return true;
+}
+
+wxString ddDatabaseDesign::getTableId(wxString tableName)
+{
+	return mappingNameToId[tableName];
+}
+
+void ddDatabaseDesign::addTableToMapping(wxString IdKey, wxString tableName)
+{
+	mappingIdToName[IdKey]=tableName;
+}
+
+wxString ddDatabaseDesign::getTableName(wxString Id)
+{
+	wxString tableName = wxEmptyString;
+	
+	tablesMappingHashMap::iterator it;
+	for (it = mappingIdToName.begin(); it != mappingIdToName.end(); ++it)
+	{
+		wxString key = it->first;
+		if(key.IsSameAs(Id,false))
+		{
+			tableName = it->second;
+			break;
+		}
+	}
+	return tableName;
+}
diff --git a/pgadmin/dd/wxhotdraw/figures/module.mk b/pgadmin/dd/wxhotdraw/figures/module.mk
index 3658c85..14ea1ce 100644
--- a/pgadmin/dd/wxhotdraw/figures/module.mk
+++ b/pgadmin/dd/wxhotdraw/figures/module.mk
@@ -28,3 +28,4 @@ EXTRA_DIST += \
 	$(srcdir)/dd/wxhotdraw/figures/module.mk
 
 include $(srcdir)/dd/wxhotdraw/figures/defaultAttributes/module.mk
+include $(srcdir)/dd/wxhotdraw/figures/xml/module.mk
diff --git a/pgadmin/dd/wxhotdraw/figures/xml/module.mk b/pgadmin/dd/wxhotdraw/figures/xml/module.mk
new file mode 100644
index 0000000..271f4f2
--- /dev/null
+++ b/pgadmin/dd/wxhotdraw/figures/xml/module.mk
@@ -0,0 +1,16 @@
+#######################################################################
+#
+# pgAdmin III - PostgreSQL Tools
+# 
+# Copyright (C) 2002 - 2011, The pgAdmin Development Team
+# This software is released under the PostgreSQL Licence
+#
+# module.mk - pgadmin/dd/wxhotdraw/figures/xml/ Makefile fragment
+#
+#######################################################################
+
+pgadmin3_SOURCES += \
+	$(srcdir)/dd/wxhotdraw/figures/xml/wxhdStorage.cpp
+
+EXTRA_DIST += \
+	$(srcdir)/dd/wxhotdraw/figures/xml/module.mk
diff --git a/pgadmin/dd/wxhotdraw/figures/xml/wxhdStorage.cpp b/pgadmin/dd/wxhotdraw/figures/xml/wxhdStorage.cpp
new file mode 100644
index 0000000..366380f
--- /dev/null
+++ b/pgadmin/dd/wxhotdraw/figures/xml/wxhdStorage.cpp
@@ -0,0 +1,34 @@
+//////////////////////////////////////////////////////////////////////////
+//
+// pgAdmin III - PostgreSQL Tools
+//
+// Copyright (C) 2002 - 2011, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+// wxhdStorage.cpp - Base class for managing all figures persistence
+//
+//////////////////////////////////////////////////////////////////////////
+
+#include "pgAdmin3.h"
+
+// libxml2 headers
+#include <libxml/xmlwriter.h>
+#include <libxml/xmlreader.h>
+
+// App headers
+#include "dd/wxhotdraw/figures/xml/wxhdStorage.h"
+
+wxhdStorage::wxhdStorage():
+	wxhdObject()
+{
+}
+
+bool wxhdStorage::Read(xmlTextReaderPtr reader)
+{
+	return false;
+}
+
+bool wxhdStorage::Write(xmlTextWriterPtr writer, wxhdIFigure *figure)
+{
+	return false;
+}
diff --git a/pgadmin/dd/wxhotdraw/main/wxhdDrawingView.cpp b/pgadmin/dd/wxhotdraw/main/wxhdDrawingView.cpp
index 1cdf9a0..11e6f43 100644
--- a/pgadmin/dd/wxhotdraw/main/wxhdDrawingView.cpp
+++ b/pgadmin/dd/wxhotdraw/main/wxhdDrawingView.cpp
@@ -344,13 +344,12 @@ void wxhdDrawingView::onMotion(wxMouseEvent &event)
 		if(event.Dragging())
 		{
 			drawingEditor->tool()->mouseDrag(ddEvent);
+			this->Refresh();			//only a dragging event on montion will change model
 		}
 		else
 		{
 			drawingEditor->tool()->mouseMove(ddEvent);
 		}
-
-		this->Refresh();
 	}
 }
 
diff --git a/pgadmin/frm/frmDatabaseDesigner.cpp b/pgadmin/frm/frmDatabaseDesigner.cpp
index ff9fdd1..c6ae257 100644
--- a/pgadmin/frm/frmDatabaseDesigner.cpp
+++ b/pgadmin/frm/frmDatabaseDesigner.cpp
@@ -16,6 +16,7 @@
 #include <wx/dcbuffer.h>
 #include <wx/aui/aui.h>
 #include <wx/splitter.h>
+#include <wx/filename.h>
 
 // App headers
 #include "frm/frmMain.h"
@@ -42,6 +43,8 @@
 #include "dd/dditems/figures/ddTableFigure.h"
 #include "dd/dditems/utilities/ddTableNameDialog.h"
 
+#include "dd/dditems/figures/xml/ddXmlStorage.h"
+
 // Icons
 #include "images/ddmodel-32.pngc"
 #include "images/file_new.pngc"
@@ -49,6 +52,8 @@
 #include "images/ddRemoveTable2.pngc"
 #include "images/continue.pngc"
 #include "images/help.pngc"
+#include "images/file_save.pngc"
+#include "images/file_open.pngc"
 
 BEGIN_EVENT_TABLE(frmDatabaseDesigner, pgFrame)
 	EVT_MENU(MNU_NEW,               frmDatabaseDesigner::OnNewModel)
@@ -56,6 +61,9 @@ BEGIN_EVENT_TABLE(frmDatabaseDesigner, pgFrame)
 	EVT_MENU(MNU_DELETETABLE,       frmDatabaseDesigner::OnDeleteTable)
 	EVT_MENU(MNU_ADDCOLUMN,         frmDatabaseDesigner::OnAddColumn)
 	EVT_MENU(MNU_GENERATEMODEL,     frmDatabaseDesigner::OnModelGeneration)
+	EVT_MENU(MNU_SAVEMODEL,			frmDatabaseDesigner::OnModelSave)
+	EVT_MENU(MNU_SAVEMODELAS,		frmDatabaseDesigner::OnModelSaveAs)
+	EVT_MENU(MNU_LOADMODEL,			frmDatabaseDesigner::OnModelLoad)
 	EVT_CLOSE(                      frmDatabaseDesigner::OnClose)
 END_EVENT_TABLE()
 
@@ -63,6 +71,7 @@ frmDatabaseDesigner::frmDatabaseDesigner(frmMain *form, const wxString &_title,
 	: pgFrame(NULL, _title)
 {
 	mainForm = form;
+	changed = false;
 	SetTitle(wxT("Database Designer"));
 	SetIcon(wxIcon(*ddmodel_32_png_ico));
 
@@ -80,7 +89,11 @@ frmDatabaseDesigner::frmDatabaseDesigner(frmMain *form, const wxString &_title,
 
 	// Set File menu
 	wxMenu *fileMenu = new wxMenu();
-	fileMenu->Append(MNU_NEW, _("&New database design\tCtrl-N"), _("Create a new database design"));
+	fileMenu->Append(MNU_NEW, _("&New model\tCtrl-N"), _("Create a new model"));
+	fileMenu->AppendSeparator();
+	fileMenu->Append(MNU_LOADMODEL, _("&Open model..."), _("Open a model"));
+	fileMenu->Append(MNU_SAVEMODEL, _("&Save model"), _("Save model"));
+	fileMenu->Append(MNU_SAVEMODELAS, _("&Save model as..."), _("Save model as..."));
 	fileMenu->AppendSeparator();
 	fileMenu->Append(MNU_EXIT, _("E&xit\tCtrl-W"), _("Exit database designer window"));
 
@@ -104,11 +117,15 @@ frmDatabaseDesigner::frmDatabaseDesigner(frmMain *form, const wxString &_title,
 	// Set toolbar
 	toolBar = new ctlMenuToolbar(this, -1, wxDefaultPosition, wxDefaultSize, wxTB_FLAT | wxTB_NODIVIDER);
 	toolBar->SetToolBitmapSize(wxSize(16, 16));
-	toolBar->AddTool(MNU_NEW, _("New"), *file_new_png_bmp, _("New database design"), wxITEM_NORMAL);
-	toolBar->AddTool(MNU_ADDTABLE, _("Add Table"), *table_png_bmp, _("Add empty table to the current model"), wxITEM_NORMAL);
-	toolBar->AddTool(MNU_DELETETABLE, _("Delete Table"), wxBitmap(*ddRemoveTable2_png_img), _("Delete selected table"), wxITEM_NORMAL);
-	toolBar->AddTool(MNU_ADDCOLUMN, _("Add Column"), *table_png_bmp, _("Add new column to the selected table"), wxITEM_NORMAL);
-	toolBar->AddTool(MNU_GENERATEMODEL, _("Generate Model"), *continue_png_bmp, _("Generate SQL for the current model"), wxITEM_NORMAL);
+	toolBar->AddTool(MNU_NEW, _("New"), *file_new_png_bmp, _("New model"), wxITEM_NORMAL);
+	toolBar->AddSeparator();
+	toolBar->AddTool(MNU_LOADMODEL, _("Open model"), *file_open_png_bmp, _("Open a model"), wxITEM_NORMAL);
+	toolBar->AddTool(MNU_SAVEMODEL, _("Save model"), *file_save_png_bmp, _("Save model"), wxITEM_NORMAL);
+	toolBar->AddSeparator();
+	toolBar->AddTool(MNU_ADDTABLE, _("Add table"), *table_png_bmp, _("Add an empty table"), wxITEM_NORMAL);
+	toolBar->AddTool(MNU_DELETETABLE, _("Delete table"), wxBitmap(*ddRemoveTable2_png_img), _("Delete selected table"), wxITEM_NORMAL);
+	toolBar->AddTool(MNU_ADDCOLUMN, _("Add column"), *table_png_bmp, _("Add a new column to the selected table"), wxITEM_NORMAL);
+	toolBar->AddTool(MNU_GENERATEMODEL, _("Generate SQL"), *continue_png_bmp, _("Generate SQL for the current model"), wxITEM_NORMAL);
 	toolBar->AddSeparator();
 	toolBar->AddTool(MNU_HELP, _("Help"), *help_png_bmp, _("Display help"), wxITEM_NORMAL);
 	toolBar->Realize();
@@ -189,6 +206,8 @@ void frmDatabaseDesigner::OnAddTable(wxCommandEvent &event)
 		                                           );
 		design->addTable(newTable);
 		design->refreshDraw();
+		changed=true;
+		setExtendedTitle();
 	}
 	delete newTableDialog;
 }
@@ -197,11 +216,13 @@ void frmDatabaseDesigner::OnDeleteTable(wxCommandEvent &event)
 {
 	ddDrawingView *v = (ddDrawingView *) design->getEditor()->view();
 	v->deleteSelectedFigures();
+	changed=true;
+	setExtendedTitle();
 }
 
 void frmDatabaseDesigner::OnAddColumn(wxCommandEvent &event)
 {
-	ddTableFigure *table = design->getSelectedTable();
+ 	ddTableFigure *table = design->getSelectedTable();
 	wxTextEntryDialog nameDialog (this, wxT("New column name"), wxT("Add a column"), wxT("NewColumn"));
 	int answer;
 	wxString tmpString;
@@ -216,8 +237,11 @@ void frmDatabaseDesigner::OnAddColumn(wxCommandEvent &event)
 			if (answer == wxID_OK)
 			{
 				tmpString = nameDialog.GetValue();
-				if(table->colNameAvailable(tmpString))
+				if(table->getColByName(tmpString)==NULL){
 					table->addColumn(new ddColumnFigure(tmpString, table));
+					changed=true;
+					setExtendedTitle();
+				}
 				else
 				{
 					wxString msg(wxT("Error trying to add new column '"));
@@ -242,6 +266,8 @@ void frmDatabaseDesigner::OnNewModel(wxCommandEvent &event)
 {
 	design->eraseModel();
 	sqltext->Clear();
+	changed=false;
+	setExtendedTitle();
 }
 
 
@@ -259,6 +285,77 @@ void frmDatabaseDesigner::OnModelGeneration(wxCommandEvent &event)
 	}
 }
 
+void frmDatabaseDesigner::OnModelSaveAs(wxCommandEvent &event)
+{
+	wxFileDialog openFileDialog( this, _("Save model"), wxEmptyString, wxEmptyString, wxT("*.pgd"),
+		                  wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
+ 
+	if ( openFileDialog.ShowModal() == wxID_OK )
+	{
+		wxString path;
+		path.append( openFileDialog.GetDirectory() );
+		path.append( wxFileName::GetPathSeparator() );
+		path.append( openFileDialog.GetFilename() );
+		if(!path.Lower().Matches(wxT("*.pgd")))
+			path.append(wxT(".pgd"));
+		lastFile = path;
+		changed=false;
+		setExtendedTitle();
+		design->writeXmlModel(path);
+	}
+}
+
+void frmDatabaseDesigner::OnModelSave(wxCommandEvent &event)
+{
+	if ( lastFile != wxEmptyString )
+	{
+		design->writeXmlModel(lastFile);
+		changed=false;
+		setExtendedTitle();
+	}
+	else
+	{
+		OnModelSaveAs(event);
+	}
+}
+
+
+void frmDatabaseDesigner::OnModelLoad(wxCommandEvent &event)
+{
+	wxFileDialog openFileDialog( this, _("Open model"), wxEmptyString, wxEmptyString, wxT("*.pgd"),
+		                  wxOPEN | wxFD_FILE_MUST_EXIST, wxDefaultPosition);
+ 
+	if ( openFileDialog.ShowModal() == wxID_OK )
+	{
+		wxString path;
+		path.append( openFileDialog.GetDirectory() );
+		path.append( wxFileName::GetPathSeparator() );
+		path.append( openFileDialog.GetFilename() );
+		if(!path.Lower().Matches(wxT("*.pgd")))
+			path.append(wxT(".pgd"));
+		design->eraseModel();
+		lastFile = path;
+		design->readXmlModel(path);
+		changed=false;
+		setExtendedTitle();
+	}
+}
+
+void frmDatabaseDesigner::setExtendedTitle()
+{	
+	wxString chgStr;
+	if (changed)
+		chgStr = wxT(" *");
+
+	wxString title = wxT("Database Design");
+
+	if (lastFile.IsEmpty())
+		SetTitle( title + chgStr);
+	else
+	{
+		SetTitle( title + wxT(" - [") + lastFile + wxT("]") + chgStr);
+	}
+}
 
 ///////////////////////////////////////////////////////
 
diff --git a/pgadmin/include/dd/dditems/figures/ddColumnFigure.h b/pgadmin/include/dd/dditems/figures/ddColumnFigure.h
index d5f2409..5a25224 100644
--- a/pgadmin/include/dd/dditems/figures/ddColumnFigure.h
+++ b/pgadmin/include/dd/dditems/figures/ddColumnFigure.h
@@ -24,6 +24,8 @@ class ddColumnFigure : public wxhdAttributeFigure
 {
 public:
 	ddColumnFigure(wxString &columnName, ddTableFigure *owner, ddRelationshipItem *sourceFk = NULL);
+	ddColumnFigure(wxString &columnName, ddTableFigure *owner, ddColumnOptionType option, bool isGenFk, bool isPkColumn, wxString colDataType, int p=-1, int s=-1, int ukIdx=-1, ddRelationshipItem *sourceFk = NULL,ddRelationshipItem *usedAsFkDestFor = NULL );
+	void Init(wxString &columnName, ddTableFigure *owner, ddRelationshipItem *sourceFk = NULL);
 	~ddColumnFigure();
 	virtual void basicMoveBy(int x, int y);
 	virtual void moveTo(int x, int y);
@@ -66,18 +68,20 @@ public:
 	void activateGenFkName();
 	void deactivateGenFkName();
 	ddRelationshipItem *getFkSource();
+	void setFkSource(ddRelationshipItem *newColumn);
 	int getPrecision();
 	int getScale();
 	void setAsUserCreatedFk(ddRelationshipItem *relatedFkItem);
 	ddRelationshipItem *getRelatedFkItem();
 	bool validateColumn(wxString &errors);
 	void setTextColour(wxColour colour);
-
-
+	ddColumnKindIcon* getKindImage() {return kindImage;};
+	ddColumnOptionIcon* getOptionImage() {return optionImage;};
+	ddTextTableItemFigure* getColumnText() {return columnText;};
 
 protected:
-	ddColumnKindIcon *leftImage;
-	ddColumnOptionIcon *centerImage;
+	ddColumnKindIcon *kindImage;
+	ddColumnOptionIcon *optionImage;
 	ddTextTableItemFigure *columnText;
 	ddTableFigure *ownerTable;
 	ddRelationshipItem *usedAsFkDestFor;
diff --git a/pgadmin/include/dd/dditems/figures/ddColumnKindIcon.h b/pgadmin/include/dd/dditems/figures/ddColumnKindIcon.h
index ede1f30..41ea633 100644
--- a/pgadmin/include/dd/dditems/figures/ddColumnKindIcon.h
+++ b/pgadmin/include/dd/dditems/figures/ddColumnKindIcon.h
@@ -54,7 +54,8 @@ public:
 	int getUniqueConstraintIndex();
 	void setUniqueConstraintIndex(int i);
 	void setRightIconForColumn();
-
+	void setPrimaryKey(bool value);
+	void setUkIndex(int ukIdx);
 protected:
 
 private:
diff --git a/pgadmin/include/dd/dditems/figures/ddRelationshipFigure.h b/pgadmin/include/dd/dditems/figures/ddRelationshipFigure.h
index 6a4b443..c693f8c 100644
--- a/pgadmin/include/dd/dditems/figures/ddRelationshipFigure.h
+++ b/pgadmin/include/dd/dditems/figures/ddRelationshipFigure.h
@@ -80,6 +80,12 @@ public:
 	ddTableFigure *getStartTable();
 	ddTableFigure *getEndTable();
 	void changeFkOSTextColor(wxColour originalColour, wxColour fkColour, bool reset = false);
+	int getUkIndex();
+	actionKind getOnUpdateAction();
+	actionKind getOnDeleteAction();
+	bool getMatchSimple();
+	columnsHashMap& getItemsHashMap() {return chm;};
+	void initRelationValues(ddTableFigure *source, ddTableFigure *destination, int ukIdx, wxString constraint, actionKind onUpdate, actionKind onDelete, bool simpleMatch, bool identifying, bool oneToMany, bool mandatory, bool fromPk);
 
 protected:
 	virtual void basicDrawSelected(wxBufferedDC &context, wxhdDrawingView *view);
@@ -87,7 +93,6 @@ protected:
 
 private:
 	virtual void OnGenericPopupClick(wxCommandEvent &event, wxhdDrawingView *view);
-	wxArrayString strings;
 	bool fkFromPk;
 	bool fkMandatory;
 	bool fkOneToMany;
diff --git a/pgadmin/include/dd/dditems/figures/ddRelationshipItem.h b/pgadmin/include/dd/dditems/figures/ddRelationshipItem.h
index ae60063..7efdc3e 100644
--- a/pgadmin/include/dd/dditems/figures/ddRelationshipItem.h
+++ b/pgadmin/include/dd/dditems/figures/ddRelationshipItem.h
@@ -5,7 +5,7 @@
 // Copyright (C) 2002 - 2011, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
-// ddRelationshipFigure.h - Figure to draw foreign keys between tables.
+// ddRelationshipFigure.h - Item to keep track of foreign keys between tables in a relationship.
 //
 //////////////////////////////////////////////////////////////////////////
 
@@ -18,17 +18,20 @@
 class ddRelationshipItem : public wxhdObject
 {
 public:
+	ddRelationshipItem();
 	ddRelationshipItem(ddRelationshipFigure *owner, ddColumnFigure *originalColumn, ddTableFigure *destination, ddColumnOptionType type, ddColumnType colType, ddColumnFigure *existingColumn = NULL);
 	~ddRelationshipItem();
+	void initRelationshipItemValues(ddRelationshipFigure *owner, ddTableFigure *destination, bool fromExistingColumn, ddColumnFigure *fkCol, ddColumnFigure *sourceCol, wxString initialColName, wxString initialAliasName );
 	wxString originalStartColName;
 	wxString originalShortName;
 	ddColumnFigure *original;
 	ddColumnFigure *fkColumn;
-	ddTableFigure *destinationTable;
+	
 
 	wxString autoGenerateNameForFk();
 	void syncAutoFkName();
 
+	ddTableFigure *getDestinationTable() {return destinationTable;};
 	bool isAutomaticallyGenerated();
 	bool relationIsIdentifying();
 	bool relationIsMandatory();
@@ -39,6 +42,7 @@ public:
 
 private:
 	bool fkIsAutoGenerated;
+	ddTableFigure *destinationTable;
 	ddRelationshipFigure *ownerRel;
 	wxString generatedName;   //avoid linux bug
 };
diff --git a/pgadmin/include/dd/dditems/figures/ddTableFigure.h b/pgadmin/include/dd/dditems/figures/ddTableFigure.h
index 3c9ed90..b64c971 100644
--- a/pgadmin/include/dd/dditems/figures/ddTableFigure.h
+++ b/pgadmin/include/dd/dditems/figures/ddTableFigure.h
@@ -24,11 +24,15 @@ class ddTableFigure : public wxhdCompositeFigure
 {
 public:
 	ddTableFigure(wxString tableName, int x, int y, wxString shortName = wxEmptyString);
+	void InitTableValues(wxArrayString UniqueKeysName, wxString primaryKeyName, int bdc, int bdi, int maxcolsi, int minidxsi, int maxidxsi, int colsrs, int colsw, int idxsrs, int idxsw);
+	void Init(wxString tableName, int x, int y, wxString shortName = wxEmptyString);
 	~ddTableFigure();
 
 	//add remove items
-	bool colNameAvailable(wxString name);
+	ddColumnFigure* getColByName(wxString name);
 	void addColumn(ddColumnFigure *column);
+	void addColumnFromStorage(ddColumnFigure *column);
+	void syncPositionsAfterLoad();
 	void removeColumn(ddColumnFigure *column);
 
 	//movement
@@ -78,6 +82,17 @@ public:
 	void updateSizeOfObservers();
 	void prepareForDeleteFkColumn(ddColumnFigure *column);
 
+	//ScrollBar persistence related
+	int getBeginDrawCols() {return beginDrawCols;};
+	int getBeginDrawIdxs() {return beginDrawIdxs;};
+	int getMaxColIndex() {return maxColIndex;};
+	int getMinIdxIndex() {return minIdxIndex;};
+	int getMaxIdxIndex() {return maxIdxIndex;};
+	int getColsRowsSize() {return colsRowsSize;};
+	int getColsWindow() {return colsWindow;};
+	int getIdxsRowsSize() {return idxsRowsSize;};
+	int getIdxsWindow() {return idxsWindow;};
+
 protected:
 	//drawing
 	virtual void basicDraw(wxBufferedDC &context, wxhdDrawingView *view);
diff --git a/pgadmin/include/dd/dditems/figures/ddTextTableItemFigure.h b/pgadmin/include/dd/dditems/figures/ddTextTableItemFigure.h
index 597ca5a..6006524 100644
--- a/pgadmin/include/dd/dditems/figures/ddTextTableItemFigure.h
+++ b/pgadmin/include/dd/dditems/figures/ddTextTableItemFigure.h
@@ -57,6 +57,7 @@ public:
 	virtual ddColumnFigure *getOwnerColumn();
 	virtual void setOwnerColumn(ddColumnFigure *column);
 	virtual void setShowDataType(bool value);
+	virtual bool getShowDataType() {return showDataType;};
 	wxhdITool *CreateFigureTool(wxhdDrawingEditor *editor, wxhdITool *defaultTool);
 	virtual void displayBoxUpdate();
 	int getTextWidth();
@@ -66,9 +67,9 @@ public:
 	void setOwnerTable(ddTableFigure *table);  //only used by columns working as table title figure for setAlias method
 	int getPrecision();
 	int getScale();
-protected:
 	void setPrecision(int value);
 	void setScale(int value);
+protected:
 	ddColumnFigure *ownerColumn;
 	ddTableFigure *ownerTable;
 	wxString colAlias;
diff --git a/pgadmin/include/dd/dditems/figures/module.mk b/pgadmin/include/dd/dditems/figures/module.mk
index 7d4fcb8..2b48a82 100644
--- a/pgadmin/include/dd/dditems/figures/module.mk
+++ b/pgadmin/include/dd/dditems/figures/module.mk
@@ -20,3 +20,5 @@ pgadmin3_SOURCES += \
 
 EXTRA_DIST += \
 	$(srcdir)/include/dd/dditems/figures/module.mk
+
+include $(srcdir)/include/dd/dditems/figures/xml/module.mk
\ No newline at end of file
diff --git a/pgadmin/include/dd/dditems/figures/xml/ddXmlStorage.h b/pgadmin/include/dd/dditems/figures/xml/ddXmlStorage.h
new file mode 100644
index 0000000..6271b31
--- /dev/null
+++ b/pgadmin/include/dd/dditems/figures/xml/ddXmlStorage.h
@@ -0,0 +1,58 @@
+//////////////////////////////////////////////////////////////////////////
+//
+// pgAdmin III - PostgreSQL Tools
+//
+// Copyright (C) 2002 - 2011, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+// wxhdAbstractFigure.h - Base class for all figures with attributes (line size, fonts and others)
+//
+//////////////////////////////////////////////////////////////////////////
+
+#ifndef DDXMLSTORAGE_H
+#define DDXMLSTORAGE_H
+
+#include "dd/wxhotdraw/figures/xml/wxhdStorage.h"
+#include "dd/ddmodel/ddDatabaseDesign.h"
+
+class ddXmlStorage : public wxhdStorage
+{
+public:
+	ddXmlStorage();
+	
+	//Generic part
+	static bool Read(xmlTextReaderPtr reader);
+	static bool Write(xmlTextWriterPtr writer, wxhdIFigure *figure);
+	static void setModel(ddDatabaseDesign *sourceDesign);
+	static wxString getModelDTD();
+
+	//Write xml info Database Designer Related
+	static void WriteLocal( xmlTextWriterPtr writer, ddColumnOptionIcon *figure);
+	static void WriteLocal( xmlTextWriterPtr writer, ddColumnKindIcon *figure);
+	static void WriteLocal( xmlTextWriterPtr writer, ddTextTableItemFigure *figure);
+	static void WriteLocal( xmlTextWriterPtr writer, ddColumnFigure *figure);
+	static void WriteLocal( xmlTextWriterPtr writer, ddTableFigure *figure);
+	static void WriteLocal( xmlTextWriterPtr writer, ddRelationshipFigure *figure);
+	static void WriteLocal( xmlTextWriterPtr writer, ddRelationshipItem *item);
+	static void StartModel( xmlTextWriterPtr writer, ddDatabaseDesign *design);
+	static void EndModel( xmlTextWriterPtr writer);
+	static bool processResult(int value);
+	
+	//Generic node processing functions
+	static wxString getNodeName(xmlTextReaderPtr reader);
+	static int getNodeType(xmlTextReaderPtr reader);
+	static wxString getNodeValue(xmlTextReaderPtr reader);
+	
+	//Read xml info Database Designer Related
+	static void selectReader(xmlTextReaderPtr reader);
+	static ddTableFigure* getTable(xmlTextReaderPtr reader);
+	static ddColumnFigure* getColumn(xmlTextReaderPtr reader, ddTableFigure *colOwner);
+	static ddRelationshipFigure* getRelationship(xmlTextReaderPtr reader);
+	static ddRelationshipItem* getRelationshipItem(xmlTextReaderPtr reader, ddRelationshipFigure *itemOwner, ddTableFigure *source, ddTableFigure *destination);
+	static void initialModelParse(xmlTextReaderPtr reader);
+	
+private:
+
+	static ddDatabaseDesign *design;
+};
+#endif
diff --git a/pgadmin/include/dd/dditems/figures/xml/module.mk b/pgadmin/include/dd/dditems/figures/xml/module.mk
new file mode 100644
index 0000000..fa106ce
--- /dev/null
+++ b/pgadmin/include/dd/dditems/figures/xml/module.mk
@@ -0,0 +1,16 @@
+#######################################################################
+#
+# pgAdmin III - PostgreSQL Tools
+# 
+# Copyright (C) 2002 - 2011, The pgAdmin Development Team
+# This software is released under the PostgreSQL Licence
+#
+# module.mk - pgadmin/include/dd/dditems/figures/xml Makefile fragment
+#
+#######################################################################
+
+pgadmin3_SOURCES += \
+	$(srcdir)/include/dd/dditems/figures/xml/ddXmlStorage.h
+
+EXTRA_DIST += \
+	$(srcdir)/include/dd/dditems/figures/xml/module.mk
diff --git a/pgadmin/include/dd/dditems/utilities/ddDataType.h b/pgadmin/include/dd/dditems/utilities/ddDataType.h
index d6099fd..19b9ea8 100644
--- a/pgadmin/include/dd/dditems/utilities/ddDataType.h
+++ b/pgadmin/include/dd/dditems/utilities/ddDataType.h
@@ -23,7 +23,11 @@ enum ddMenuColumn
 enum ddHiddenTypes
 {
 	DDTABLEFIGURE = 100,
-	DDRELATIONSHIPFIGURE = 200
+	DDRELATIONSHIPFIGURE = 200,
+	DDCOLUMNFIGURE=300,
+	DDCOLUMNOPTIONICON=400,
+	DDCOLUMNKINDICON=500,
+	DDTEXTTABLEITEMFIGURE=600
 };
 
 enum ddDataType
diff --git a/pgadmin/include/dd/ddmodel/ddDatabaseDesign.h b/pgadmin/include/dd/ddmodel/ddDatabaseDesign.h
index 3eda568..ade1f93 100644
--- a/pgadmin/include/dd/ddmodel/ddDatabaseDesign.h
+++ b/pgadmin/include/dd/ddmodel/ddDatabaseDesign.h
@@ -12,10 +12,14 @@
 #ifndef DDDATABASEDESIGN_H
 #define DDDATABASEDESIGN_H
 
+#include <libxml/xmlwriter.h>
+
 #include "dd/wxhotdraw/main/wxhdDrawingEditor.h"
 #include "dd/wxhotdraw/tools/wxhdITool.h"
 #include "dd/dditems/figures/ddTableFigure.h"
 
+WX_DECLARE_STRING_HASH_MAP( wxString , tablesMappingHashMap );
+
 class ddDatabaseDesign : public wxObject
 {
 public:
@@ -32,13 +36,21 @@ public:
 	bool validateModel(wxString &errors);
 	wxString getNewTableName();
 	ddTableFigure *getSelectedTable();
-	bool containsTable(wxString tableName);
+	ddTableFigure *getTable(wxString tableName);
+	bool writeXmlModel(wxString file);
+	bool readXmlModel(wxString file);
 
+	wxString getTableId(wxString tableName);
+	void addTableToMapping(wxString IdKey, wxString tableName);
+	wxString getTableName(wxString Id);
 
 protected:
-
+	tablesMappingHashMap mappingNameToId;
+	tablesMappingHashMap mappingIdToName;
 private:
 	wxhdDrawingEditor *draw;
 	wxhdITool *tool;
+	xmlTextWriterPtr xmlWriter;
+
 };
 #endif
diff --git a/pgadmin/include/dd/ddmodel/ddDrawingView.h b/pgadmin/include/dd/ddmodel/ddDrawingView.h
index e02a298..1629a7c 100644
--- a/pgadmin/include/dd/ddmodel/ddDrawingView.h
+++ b/pgadmin/include/dd/ddmodel/ddDrawingView.h
@@ -10,7 +10,7 @@
 //////////////////////////////////////////////////////////////////////////
 
 #ifndef DDDRAWINGVIEW_H
-#define DDDRAWINGVIEW
+#define DDDRAWINGVIEW_H
 
 #include "dd/wxhotdraw/main/wxhdDrawingView.h"
 
diff --git a/pgadmin/include/dd/wxhotdraw/figures/module.mk b/pgadmin/include/dd/wxhotdraw/figures/module.mk
index c1699db..0028cb9 100644
--- a/pgadmin/include/dd/wxhotdraw/figures/module.mk
+++ b/pgadmin/include/dd/wxhotdraw/figures/module.mk
@@ -28,3 +28,4 @@ EXTRA_DIST += \
 	$(srcdir)/include/dd/wxhotdraw/figures/module.mk
 
 include $(srcdir)/include/dd/wxhotdraw/figures/defaultAttributes/module.mk
+include $(srcdir)/include/dd/wxhotdraw/figures/xml/module.mk
\ No newline at end of file
diff --git a/pgadmin/include/dd/wxhotdraw/figures/xml/module.mk b/pgadmin/include/dd/wxhotdraw/figures/xml/module.mk
new file mode 100644
index 0000000..123d228
--- /dev/null
+++ b/pgadmin/include/dd/wxhotdraw/figures/xml/module.mk
@@ -0,0 +1,16 @@
+#######################################################################
+#
+# pgAdmin III - PostgreSQL Tools
+# 
+# Copyright (C) 2002 - 2011, The pgAdmin Development Team
+# This software is released under the PostgreSQL Licence
+#
+# module.mk - pgadmin/include/dd/wxhotdraw/figures/xml Makefile fragment
+#
+#######################################################################
+
+pgadmin3_SOURCES += \
+	$(srcdir)/include/dd/wxhotdraw/figures/xml/wxhdStorage.h
+
+EXTRA_DIST += \
+	$(srcdir)/include/dd/wxhotdraw/figures/xml/module.mk
diff --git a/pgadmin/include/dd/wxhotdraw/figures/xml/wxhdStorage.h b/pgadmin/include/dd/wxhotdraw/figures/xml/wxhdStorage.h
new file mode 100644
index 0000000..42705f9
--- /dev/null
+++ b/pgadmin/include/dd/wxhotdraw/figures/xml/wxhdStorage.h
@@ -0,0 +1,25 @@
+//////////////////////////////////////////////////////////////////////////
+//
+// pgAdmin III - PostgreSQL Tools
+//
+// Copyright (C) 2002 - 2011, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+// wxhdAbstractFigure.h - Base class for all figures with attributes (line size, fonts and others)
+//
+//////////////////////////////////////////////////////////////////////////
+
+#ifndef WXHDSTORAGE_H
+#define WXHDSTORAGE_H
+
+#include "dd/wxhotdraw/main/wxhdObject.h"
+#include "dd/wxhotdraw/figures/wxhdIFigure.h"
+
+class wxhdStorage : public wxhdObject
+{
+public:
+	wxhdStorage();
+	static bool Read(xmlTextReaderPtr reader);
+	static bool Write(xmlTextWriterPtr writer, wxhdIFigure *figure);
+};
+#endif
diff --git a/pgadmin/include/frm/frmDatabaseDesigner.h b/pgadmin/include/frm/frmDatabaseDesigner.h
index 2641121..d78f30b 100644
--- a/pgadmin/include/frm/frmDatabaseDesigner.h
+++ b/pgadmin/include/frm/frmDatabaseDesigner.h
@@ -32,16 +32,22 @@ public:
 	~frmDatabaseDesigner();
 	void Go();
 private:
+	bool changed;
+	wxString lastFile;
 	frmMain *mainForm;
 	pgConn *connection;
 	ddDatabaseDesign *design;
 	wxTextCtrl *sqltext;
+	void setExtendedTitle();
 	void OnClose(wxCloseEvent &event);
 	void OnAddTable(wxCommandEvent &event);
 	void OnDeleteTable(wxCommandEvent &event);
 	void OnAddColumn(wxCommandEvent &event);
 	void OnNewModel(wxCommandEvent &event);
 	void OnModelGeneration(wxCommandEvent &event);
+	void OnModelSaveAs(wxCommandEvent &event);
+	void OnModelSave(wxCommandEvent &event);
+	void OnModelLoad(wxCommandEvent &event);
 	wxAuiManager manager;
 	DECLARE_EVENT_TABLE()
 };
diff --git a/pgadmin/include/frm/menu.h b/pgadmin/include/frm/menu.h
index f7c1dc4..ef0d742 100644
--- a/pgadmin/include/frm/menu.h
+++ b/pgadmin/include/frm/menu.h
@@ -111,6 +111,9 @@ enum
 	MNU_ADDTABLE,
 	MNU_DELETETABLE,
 	MNU_GENERATEMODEL,
+	MNU_SAVEMODEL,
+	MNU_LOADMODEL,
+	MNU_SAVEMODELAS,
 
 	MNU_RECENT,
 	MNU_NEW = MNU_RECENT + 15, // leave space for recent file entries
diff --git a/pgadmin/pgAdmin3.vcproj b/pgadmin/pgAdmin3.vcproj
index 33b5245..3565a00 100644
--- a/pgadmin/pgAdmin3.vcproj
+++ b/pgadmin/pgAdmin3.vcproj
@@ -4030,6 +4030,14 @@
 							RelativePath=".\include\dd\dditems\figures\module.mk"
 							>
 						</File>
+						<Filter
+							Name="xml"
+							>
+							<File
+								RelativePath=".\include\dd\dditems\figures\xml\ddXmlStorage.h"
+								>
+							</File>
+						</Filter>
 					</Filter>
 					<Filter
 						Name="handles"
@@ -4274,6 +4282,14 @@
 								>
 							</File>
 						</Filter>
+						<Filter
+							Name="xml"
+							>
+							<File
+								RelativePath=".\include\dd\wxhotdraw\figures\xml\wxhdStorage.h"
+								>
+							</File>
+						</Filter>
 					</Filter>
 					<Filter
 						Name="handles"
@@ -6015,6 +6031,14 @@
 						RelativePath=".\dd\dditems\figures\module.mk"
 						>
 					</File>
+					<Filter
+						Name="xml"
+						>
+						<File
+							RelativePath=".\dd\dditems\figures\xml\ddXmlStorage.cpp"
+							>
+						</File>
+					</Filter>
 				</Filter>
 				<Filter
 					Name="handles"
@@ -6255,6 +6279,14 @@
 							>
 						</File>
 					</Filter>
+					<Filter
+						Name="xml"
+						>
+						<File
+							RelativePath=".\dd\wxhotdraw\figures\xml\wxhdStorage.cpp"
+							>
+						</File>
+					</Filter>
 				</Filter>
 				<Filter
 					Name="handles"
